import pytest

from scout.models.hgnc_map import HgncGene

from scout.exceptions import IntegrityError

#################### HGNC gene tests ####################
def test_insert_gene(adapter, parsed_gene):
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting a gene
    
    gene_obj = HgncGene(**parsed_gene)
    
    obj_id = adapter.load_hgnc_gene(gene_obj)

    ##THEN assert that the gene is there
    assert adapter.all_genes().count() == 1
    ##THEN assert that no genes are in the '38' build
    assert adapter.all_genes(build='38').count() == 0

def test_insert_many_genes(adapter, parsed_gene):
    adapter = adapter
    gene_objs = []
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0
    
    ##WHEN inserting a bulk of genes
    for i in range(300):
        parsed_gene['hgnc_id'] = i
        gene_objs.append(HgncGene(**parsed_gene))

    result = adapter.load_hgnc_bulk(gene_objs)

    ##THEN assert that the genes are loaded
    assert adapter.all_genes().count() == 300
    ##THEN assert that no genes are in the '38' build
    assert adapter.all_genes(build='38').count() == 0

def test_insert_bulk(adapter, gene_bulk):
    ## GIVEN an empty adapter and a bulk of genes
    adapter = adapter
    
    assert adapter.all_genes().count() == 0
    assert len(gene_bulk) > 0
    
    ## WHEN inserting the gene objects
    result = adapter.load_hgnc_bulk(gene_bulk)
    
    ##THEN assert that the genes are loaded
    assert adapter.all_genes().count() == len(gene_bulk)
    
    
def test_insert_many_genes_duplicate(adapter):
    adapter = adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    gene_objs = []
    gene_obj = {
        '_id': 1,
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37'
    }

    ##WHEN inserting a bulk of genes with same _id
    for i in range(300):
        gene_objs.append(gene_obj)

    ##THEN assert that IntegrityError is raised
    with pytest.raises(IntegrityError):
        result = adapter.load_hgnc_bulk(gene_objs)


def test_get_gene(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting a gene and fetching it
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37'
    }
    adapter.load_hgnc_gene(gene_obj)


    ##THEN assert that the correct gene was fetched
    res = adapter.hgnc_gene(
            hgnc_identifier=gene_obj['hgnc_id'],
            build=gene_obj['build']
        )

    assert res['hgnc_id'] == gene_obj['hgnc_id']

    ##THEN assert that there are no genes in the 38 build
    res = adapter.hgnc_gene(
            hgnc_identifier=gene_obj['hgnc_id'],
            build='38'
        )

    assert res == None

def test_get_genes(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.hgnc_genes(hgnc_symbol=gene_obj['hgnc_symbol'])

    ##THEN assert that the correct gene was fetched

    for result in res:
        assert result['hgnc_id'] == gene_obj['hgnc_id']

def test_get_genes_alias(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.hgnc_genes(hgnc_symbol='AAB')

    ##THEN assert that the correct gene was fetched

    for result in res:
        assert result['hgnc_id'] == 1

def test_get_genes_regex(real_adapter):
    adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'BCD'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }
    adapter.load_hgnc_gene(gene_obj2)

    gene_obj3 = {
        'hgnc_id': 3,
        'hgnc_symbol': 'AB',
        'build': '37',
        'aliases': ['C', 'AC'],
    }

    adapter.load_hgnc_gene(gene_obj3)

    ##THEN assert that only the correct gene was fetched for a full match
    res = adapter.hgnc_genes(hgnc_symbol='AA', search=True)
    assert res.count() == 1

    ##THEN assert that the correct gene was fetched
    res = adapter.hgnc_genes(hgnc_symbol='AB', search=True)
    assert res.count() == 1

    ##THEN assert that the correct gene was fetched
    res = adapter.hgnc_genes(hgnc_symbol='K', search=True)
    assert res.count() == 0

    ##THEN assert that the correct gene was fetched
    res = adapter.hgnc_genes(hgnc_symbol='A', search=True)
    assert res.count() == 3

    #This will only work with a real pymongo adapter
    ##THEN assert that the correct gene was fetched
    res = adapter.hgnc_genes(hgnc_symbol='a', search=True)
    assert res.count() == 3

def test_get_all_genes(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.all_genes()

    ##THEN assert that the correct number of genes where fetched
    assert res.count() == 2

def test_drop_genes(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.all_genes()
    assert res.count() == 2

    ##THEN assert that the correct number of genes where fetched
    adapter.drop_genes()
    res = adapter.all_genes()
    assert res.count() == 0

def test_hgncid_to_gene(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.all_genes()
    assert res.count() == 2

    ##THEN assert that the correct number of genes where fetched
    res = adapter.hgncid_to_gene()
    assert gene_obj['hgnc_id'] in res
    assert gene_obj2['hgnc_id'] in res

    first_gene = res[gene_obj['hgnc_id']]

    assert first_gene['hgnc_id'] == gene_obj['hgnc_id']


def test_hgnc_symbol_to_gene(adapter):
    # adapter = real_adapter
    ##GIVEN a empty adapter
    assert adapter.all_genes().count() == 0

    ##WHEN inserting two genes and fetching one
    gene_obj = {
        'hgnc_id': 1,
        'hgnc_symbol': 'AAA',
        'build': '37',
        'aliases': ['AAA', 'AAB'],
    }
    adapter.load_hgnc_gene(gene_obj)

    gene_obj2 = {
        'hgnc_id': 2,
        'hgnc_symbol': 'AA',
        'build': '37',
        'aliases': ['AA', 'AB'],
    }

    adapter.load_hgnc_gene(gene_obj2)

    res = adapter.all_genes()
    assert res.count() == 2

    ##THEN assert that the correct number of genes where fetched
    res = adapter.hgncsymbol_to_gene()
    assert gene_obj['hgnc_symbol'] in res
    assert gene_obj2['hgnc_symbol'] in res


#################### HGNC transcript tests ####################

def test_insert_transcript(adapter):
    ##GIVEN a empty adapter
    assert adapter.transcripts().count() == 0

    ##WHEN inserting a transcript
    transcript_obj = {
        'ensembl_transcript_id': 'ENST01', # required
        'refseq_id': 'NM_1',
        'start': 1, # required
        'end': 10, # required
        'is_primary': False,
        'build': '37',
    }
    obj_id = adapter.load_hgnc_transcript(transcript_obj)

    ##THEN assert that the transcript is there
    assert adapter.transcripts().count() == 1
    ##THEN assert that no transcripts are in the '38' build
    assert adapter.transcripts(build='38').count() == 0

def test_insert_many_transcripts(adapter):
    adapter = adapter
    ##GIVEN a empty adapter
    assert adapter.transcripts().count() == 0

    transcript_objs = []
    ##WHEN inserting a bulk of transcripts
    for i in range(300):
        transcript_objs.append({
            'ensembl_transcript_id': 'ENST01', # required
            'refseq_id': 'NM_1',
            'start': 1, # required
            'end': 10, # required
            'is_primary': False,
            'build': '37',
        })

    result = adapter.load_transcript_bulk(transcript_objs)

    ##THEN assert that the transcripts are loaded
    assert adapter.transcripts().count() == 300
    ##THEN assert that no transcripts are in the '38' build
    assert adapter.transcripts(build='38').count() == 0

def test_insert_many_transcripts_duplicate(adapter):
    adapter = adapter
    ##GIVEN a empty adapter

    transcript_objs = []
    transcript_obj = {
        '_id': 1,
        'ensembl_transcript_id': 'ENST01', # required
        'refseq_id': 'NM_1',
        'start': 1, # required
        'end': 10, # required
        'is_primary': False,
        'build': '37',
    }

    ##WHEN inserting a bulk of transcripts with same _id
    for i in range(300):
        transcript_objs.append(transcript_obj)

    ##THEN assert that IntegrityError is raised
    with pytest.raises(IntegrityError):
        result = adapter.load_transcript_bulk(transcript_objs)

def test_insert_transcripts(adapter, transcript_objs):
    adapter = adapter
    ##GIVEN a empty adapter
    assert adapter.transcripts().count() == 0

    ##WHEN inserting a bulk of transcripts
    result = adapter.load_transcript_bulk(transcript_objs)

    ##THEN assert that the transcripts are loaded
    assert adapter.transcripts().count() == len(transcript_objs)
    ##THEN assert that no transcripts are in the '38' build
    assert adapter.transcripts(build='38').count() == 0

#################### Combined transcript/gene tests ####################

def test_insert_transcript(adapter):
    ##GIVEN a empty adapter
    assert adapter.transcripts().count() == 0
    assert adapter.all_genes().count() == 0

    ##WHEN inserting a gene and some transcripts
    hgnc_id = 257
    hgnc_symbol = 'ADK'
    ens_gene_id = 'ENSG00000156110'
    chrom = '10'
    build = '37'
    
    
    gene_obj  = dict(
        hgnc_id = hgnc_id,
        hgnc_symbol = hgnc_symbol,
        ensembl_id = ens_gene_id,
        chromosome = chrom,
        start = 74151185,
        end = 74709303,
        build = '37'
    )
    
    refseq_id = 'NM_006721'
    transcript_obj = {
        'hgnc_id': hgnc_id,
        'ensembl_transcript_id': 'ENST01', # required
        'refseq_id': refseq_id,
        'start': 74151185, # required
        'end': 74709303, # required
        'is_primary': True,
        'build': build,
    }
    gene_res = adapter.load_hgnc_gene(gene_obj)
    obj_id = adapter.load_hgnc_transcript(transcript_obj)

    ##THEN assert that when fetching the gene the transcript is added
    
    gene_res = adapter.hgnc_gene(hgnc_id, build=build)
    
    assert gene_res['hgnc_id'] == hgnc_id
    assert len(gene_res['transcripts']) == 1
    assert gene_res['transcripts'][0]['refseq_id'] == refseq_id
    




# -*- coding: utf-8 -*-
from flask import abort, Blueprint, request, jsonify, redirect, url_for

from scout.server.extensions import store
from scout.server.utils import templated, public_endpoint
from . import controllers

hpo_bp = Blueprint('phenotypes', __name__, template_folder='templates')

@hpo_bp.route('/phenotypes', methods=['POST', 'GET'])
@templated('phenotypes/hpo_terms.html')
def hpo_terms():
    """Render search box and view for HPO phenotype terms"""
    if request.method == 'GET':
        data = controllers.hpo_terms(store= store, limit=100)
        return data
    else: # POST. user is searching for a specific term or phenotype
        search_term = request.form.get('hpo_term')
        limit = request.form.get('limit')
        data = controllers.hpo_terms(store= store,  query = search_term, limit=limit)
        return dict(data, query=search_term, limit=limit)



from scout.utils.link import link_genes
from pprint import pprint as pp

def test_link_genes(genes37_handle, hgnc_handle, exac_handle, 
                    mim2gene_handle, genemap_handle, hpo_genes_handle):
    """docstring for test_link_genes"""
    genes = link_genes(
        ensembl_lines=genes37_handle, 
        hgnc_lines=hgnc_handle, 
        exac_lines=exac_handle, 
        mim2gene_lines=mim2gene_handle,
        genemap_lines=genemap_handle,
        hpo_lines=hpo_genes_handle,
    )
    for hgnc_id in genes:
        gene_obj = genes[hgnc_id]
        assert gene_obj['hgnc_symbol']
        assert gene_obj['hgnc_id']
        assert gene_obj['chromosome']
        assert gene_obj['start']
        assert gene_obj['end']

        assert gene_obj['hgnc_symbol'] in gene_obj['previous_symbols']



from scout.parse.hpo import (parse_hpo_disease, parse_hpo_diseases)

def test_parse_disease_lines():
    """Test how the parser behaves"""
    disease_lines = [
        "#Format: diseaseId<tab>gene-id<tab>gene-symbol",
        "OMIM:139500",
        "ORPHANET:1228",
        "DECIPHER:74",
        "OMIM:214500\t1130\tLYST",
    ]
    ## GIVEN a disease term without gene symbol
    ## WHEN parsing the line
    disease = parse_hpo_disease(disease_lines[1])
    ## THEN assert the info is correct
    assert disease['source'] == "OMIM"
    assert disease['disease_nr'] == 139500
    assert disease['hgnc_symbol'] == None

    ## GIVEN a disease term without gene symbol
    ## WHEN parsing the line
    disease = parse_hpo_disease(disease_lines[2])
    ## THEN assert the info is correct
    assert disease['source'] == "ORPHANET"
    assert disease['disease_nr'] == 1228
    assert disease['hgnc_symbol'] == None

    ## GIVEN a disease term without gene symbol
    ## WHEN parsing the line
    disease = parse_hpo_disease(disease_lines[3])
    ## THEN assert the info is correct
    assert disease['source'] == "DECIPHER"
    assert disease['disease_nr'] == 74
    assert disease['hgnc_symbol'] == None

    ## GIVEN a disease term without gene symbol
    ## WHEN parsing the line
    disease = parse_hpo_disease(disease_lines[4])
    ## THEN assert the info is correct
    assert disease['source'] == "OMIM"
    assert disease['disease_nr'] == 214500
    assert disease['hgnc_symbol'] == "LYST"

def test_parse_diseases_lines():
    ## GIVEN a iterable of disease lines
    disease_lines = [
        "#Format: diseaseId<tab>gene-id<tab>gene-symbol",
        "OMIM:300818	5277	PIGA",
        "OMIM:300868	5277	PIGA",
        "ORPHANET:447	5277	PIGA",
        "OMIM:101400	2263	FGFR2",
        "OMIM:101400	7291	TWIST1",
        "OMIM:139500",
    ]
    ## WHEN parsing the diseases
    diseases = parse_hpo_diseases(disease_lines)
    ## THEN assert that the diseases are parsed correct
    assert diseases["OMIM:300818"]['source'] == 'OMIM'
    assert diseases["OMIM:300818"]['hgnc_symbols'] == set(['PIGA'])

    assert diseases["ORPHANET:447"]['source'] == "ORPHANET"
    assert diseases["ORPHANET:447"]['hgnc_symbols'] == set(['PIGA'])

    assert diseases["OMIM:101400"]['source'] == 'OMIM'
    assert diseases["OMIM:101400"]['hgnc_symbols'] == set(['FGFR2','TWIST1'])

    assert diseases["OMIM:139500"]['source'] == 'OMIM'
    assert diseases["OMIM:139500"]['hgnc_symbols'] == set([])

def test_parse_diseases(hpo_disease_handle):
    ## GIVEN a iterable of disease lines
    ## WHEN parsing the diseases
    diseases = parse_hpo_diseases(hpo_disease_handle)
    ## THEN assert that the diseases are parsed correct
    for disease_id in diseases:
        source = disease_id.split(':')[0]
        disease_nr = int(disease_id.split(':')[1])
        
        disease_term = diseases[disease_id]
        assert disease_term['source'] == source
        assert disease_term['disease_nr'] == disease_nr



# -*- coding: utf-8 -*-
import datetime as dt
import operator
import logging

from flask import flash

from scout.parse.panel import parse_genes
from scout.build.panel import build_panel

log = logging.getLogger(__name__)

INHERITANCE_MODELS = ['ar', 'ad', 'mt', 'xr', 'xd', 'x', 'y']


def panel(store, panel_obj):
    """Preprocess a panel of genes."""
    panel_obj['institute'] = store.institute(panel_obj['institute'])
    full_name = "{} ({})".format(panel_obj['display_name'], panel_obj['version'])
    panel_obj['name_and_version'] = full_name
    return dict(panel=panel_obj)


def existing_gene(store, panel_obj, hgnc_id):
    """Check if gene is already added to a panel."""
    existing_genes = {gene['hgnc_id']: gene for gene in panel_obj['genes']}
    return existing_genes.get(hgnc_id)


def update_panel(store, panel_name, csv_lines, option):
    """Update an existing gene panel with genes.

    Args:
        store(scout.adapter.MongoAdapter)
        panel_name(str)
        csv_lines(iterable(str)): Stream with genes
        option(str): 'add' or 'replace'

    Returns:
        panel_obj(dict)
    """
    new_genes= []
    panel_obj = store.gene_panel(panel_name)
    if panel_obj is None:
        return None
    try:
        new_genes = parse_genes(csv_lines) # a list of gene dictionaries containing gene info
    except SyntaxError as error:
        flash(error.args[0], 'danger')
        return None

    # if existing genes are to be replaced by those in csv_lines
    if option == 'replace':
        # all existing genes should be deleted
        for gene in panel_obj['genes']:
            #create extra key to use in pending actions:
            gene['hgnc_symbol'] = gene['symbol']
            store.add_pending(panel_obj, gene, action='delete', info=None)

    for new_gene in new_genes:
        if not new_gene['hgnc_id']:
            flash("gene missing hgnc id: {}".format(new_gene['hgnc_symbol']),'danger')
            continue
        gene_obj = store.hgnc_gene(new_gene['hgnc_id'])
        if gene_obj is None:
            flash("gene not found: {} - {}".format(new_gene['hgnc_id'], new_gene['hgnc_symbol']),'danger')
            continue
        if new_gene['hgnc_symbol'] and gene_obj['hgnc_symbol'] != new_gene['hgnc_symbol']:
            flash("symbol mis-match: {0} | {1}".format(
                gene_obj['hgnc_symbol'], new_gene['hgnc_symbol']), 'warning')

        info_data = {
            'disease_associated_transcripts': new_gene['transcripts'],
            'reduced_penetrance': new_gene['reduced_penetrance'],
            'mosaicism': new_gene['mosaicism'],
            'inheritance_models': new_gene['inheritance_models'],
            'database_entry_version': new_gene['database_entry_version'],
        }
        if option == 'replace': # there will be no existing genes for sure, because we're replacing them all
            action = 'add'
        else: # add option. Add if genes is not existing. otherwise edit it
            existing_genes = {gene['hgnc_id'] for gene in panel_obj['genes']}
            action = 'edit' if gene_obj['hgnc_id'] in existing_genes else 'add'
        store.add_pending(panel_obj, gene_obj, action=action, info=info_data)

    return panel_obj


def new_panel(store, institute_id, panel_name, display_name, csv_lines):
    """Create a new gene panel.

    Args:
        store(scout.adapter.MongoAdapter)
        institute_id(str)
        panel_name(str)
        display_name(str)
        csv_lines(iterable(str)): Stream with genes

    Returns:
        panel_id: the ID of the new panel document created or None

    """
    institute_obj = store.institute(institute_id)
    if institute_obj is None:
        flash("{}: institute not found".format(institute_id))
        return None

    panel_obj = store.gene_panel(panel_name)
    if panel_obj:
        flash("panel already exists: {} - {}".format(panel_obj['panel_name'],
                                                     panel_obj['display_name']))
        return None

    log.debug("parse genes from CSV input")
    try:
        new_genes = parse_genes(csv_lines)
    except SyntaxError as error:
        flash(error.args[0], 'danger')
        return None

    log.debug("build new gene panel")

    panel_id = None
    try:
        panel_data = build_panel(dict(
            panel_name=panel_name,
            institute=institute_obj['_id'],
            version=1.0,
            date=dt.datetime.now(),
            display_name=display_name,
            genes=new_genes,
        ), store)
        panel_id= store.add_gene_panel(panel_data)

    except Exception as err:
        log.error('An error occurred while adding the gene panel {}'.format(err))

    return panel_id


def panel_export(store, panel_obj):
    """Preprocess a panel of genes."""
    panel_obj['institute'] = store.institute(panel_obj['institute'])
    full_name = "{}({})".format(panel_obj['display_name'], panel_obj['version'])
    panel_obj['name_and_version'] = full_name

    return dict(panel=panel_obj)



# -*- coding: utf-8 -*-
from flask_wtf import FlaskForm
from wtforms import (BooleanField, TextField, SelectMultipleField, TextField)

from scout.constants import GENETIC_MODELS


class PanelGeneForm(FlaskForm):
    disease_associated_transcripts = SelectMultipleField('Disease transcripts', choices=[])
    reduced_penetrance = BooleanField()
    mosaicism = BooleanField()
    database_entry_version = TextField()
    inheritance_models = SelectMultipleField(choices=GENETIC_MODELS)
    comment = TextField()



import logging

LOG = logging.getLogger(__name__)

def export_genes(adapter, build='37'):
    """Export all genes from the database"""
    LOG.info("Exporting all genes to .bed format")
    
    for gene_obj in adapter.all_genes(build=build):
        yield gene_obj
    


# -*- coding: utf-8 -*-
import os.path
import shutil
import datetime

import zipfile
import io
import pathlib

import logging

from operator import itemgetter

from flask import (abort, Blueprint, current_app, redirect, render_template,
                   request, url_for, send_from_directory, jsonify, Response, flash, send_file)
from flask_login import current_user
from flask_weasyprint import HTML, render_pdf
from werkzeug.datastructures import Headers
from dateutil.parser import parse as parse_date
from scout.constants import CLINVAR_HEADER, CASEDATA_HEADER
from scout.server.extensions import store, mail
from scout.server.utils import templated, institute_and_case, user_institutes
from . import controllers

log = logging.getLogger(__name__)

cases_bp = Blueprint('cases', __name__, template_folder='templates',
                     static_folder='static', static_url_path='/cases/static')


@cases_bp.route('/institutes')
@templated('cases/index.html')
def index():
    """Display a list of all user institutes."""
    institute_objs = user_institutes(store, current_user)
    institutes_count = ((institute_obj, store.cases(collaborator=institute_obj['_id']).count())
                        for institute_obj in institute_objs if institute_obj)
    return dict(institutes=institutes_count)


@cases_bp.route('/<institute_id>')
@templated('cases/cases.html')
def cases(institute_id):
    """Display a list of cases for an institute."""
    institute_obj = institute_and_case(store, institute_id)
    query = request.args.get('query')

    limit = 100
    if request.args.get('limit'):
        limit = int(request.args.get('limit'))

    skip_assigned = request.args.get('skip_assigned')
    all_cases = store.cases(institute_id, name_query=query, skip_assigned=skip_assigned)
    data = controllers.cases(store, all_cases, limit)

    sanger_unevaluated = controllers.get_sanger_unevaluated(store, institute_id, current_user.email)
    if len(sanger_unevaluated)> 0:
        data['sanger_unevaluated'] = sanger_unevaluated

    return dict(institute=institute_obj, skip_assigned=skip_assigned, query=query, **data)


@cases_bp.route('/<institute_id>/<case_name>')
@templated('cases/case.html')
def case(institute_id, case_name):
    """Display one case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    data = controllers.case(store, institute_obj, case_obj)
    return dict(institute=institute_obj, case=case_obj, **data)


@cases_bp.route('/<institute_id>/clinvar_submissions', methods=['GET','POST'])
@templated('cases/clinvar_submissions.html')
def clinvar_submissions(institute_id):

    def generate_csv(header, lines):
        yield header + '\n'
        for line in lines: # lines have already quoted fields
            yield line + '\n'

    if request.method == 'POST':
        submission_id = request.form.get('submission_id')
        if request.form.get('update_submission'):
            if request.form.get('update_submission') == 'close': # close a submission
                store.update_clinvar_submission_status(current_user.email, submission_id, 'closed')
            elif request.form.get('update_submission') == 'open':
                store.update_clinvar_submission_status(current_user.email, submission_id, 'open') # open a submission
            elif request.form.get('update_submission') == 'register_id' and request.form.get('clinvar_id'): # provide an official clinvar submission ID
                result = store.update_clinvar_id(clinvar_id = request.form.get('clinvar_id'), submission_id = submission_id)
            elif request.form.get('update_submission') == 'delete': # delete a submission
                deleted_objects, deleted_submissions = store.delete_submission(submission_id = submission_id)
                flash("Removed {} objects and {} submission from database".format(deleted_objects, deleted_submissions), 'info')
        elif request.form.get('delete_variant'): # delete a variant from a submission
            store.delete_clinvar_object(object_id = request.form.get('delete_variant'), object_type='variant_data', submission_id = submission_id) # remove variant and associated_casedata
        elif request.form.get('delete_casedata'): # delete a case from a submission
            store.delete_clinvar_object(object_id = request.form.get('delete_casedata'), object_type='case_data', submission_id = submission_id) # remove just the casedata associated to a variant
        else: # Download submission CSV files (for variants or casedata)

            clinvar_subm_id = request.form.get('clinvar_id')
            if clinvar_subm_id =='':
                flash('In order to download a submission CSV file you should register a Clinvar submission Name first!','warning')
                return redirect(request.referrer)

            csv_type = ''
            csv_type = request.form.get('csv_type')

            submission_objs = store.clinvar_objs(submission_id=submission_id, key_id=csv_type) # a list of clinvar submission objects (variants or casedata)
            if submission_objs:
                csv_header_obj = controllers.clinvar_header(submission_objs, csv_type) # custom csv header (dict as in constants CLINVAR_HEADER and CASEDATA_HEADER, but with required fields only)
                csv_lines = controllers.clinvar_lines(submission_objs, csv_header_obj) # csv lines (one for each variant/casedata to be submitted)
                csv_header = list(csv_header_obj.values())
                csv_header = ['"'+str(x)+'"' for x in csv_header] # quote columns in header for csv rendering

                headers = Headers()
                headers.add('Content-Disposition','attachment', filename=clinvar_subm_id+'_'+str(datetime.datetime.now().strftime("%Y-%m-%d"))+'.csv')
                return Response(generate_csv(','.join(csv_header), csv_lines), mimetype='text/csv', headers=headers)
            else:
                flash('There are no submission objects of type "{}" to include in the csv file!'.format(csv_type),'warning')

    data = {
        'submissions' : controllers.clinvar_submissions(store, current_user.email, institute_id),
        'institute_id' : institute_id,
        'variant_header_fields' : CLINVAR_HEADER ,
        'casedata_header_fields' : CASEDATA_HEADER
    }
    return data


@cases_bp.route('/<institute_id>/causatives')
@templated('cases/causatives.html')
def causatives(institute_id):
    institute_obj = institute_and_case(store, institute_id)
    variants = store.check_causatives(institute_obj=institute_obj)
    all_variants = {}
    all_cases = {}
    for variant_obj in variants:
        if variant_obj['case_id'] not in all_cases:
            case_obj = store.case(variant_obj['case_id'])
            all_cases[variant_obj['case_id']] = case_obj
        else:
            case_obj = all_cases[variant_obj['case_id']]

        if variant_obj['variant_id'] not in all_variants:
            all_variants[variant_obj['variant_id']] = []
        all_variants[variant_obj['variant_id']].append((case_obj, variant_obj))

    return dict(institute=institute_obj, variant_groups=all_variants)


@cases_bp.route('/<institute_id>/<case_name>/synopsis', methods=['POST'])
def case_synopsis(institute_id, case_name):
    """Update (PUT) synopsis of a specific case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    new_synopsis = request.form.get('synopsis')
    controllers.update_synopsis(store, institute_obj, case_obj, user_obj, new_synopsis)
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/case_report', methods=['GET'])
@templated('cases/case_report.html')
def case_report(institute_id, case_name):
    """Visualize case report"""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    data = controllers.case_report_content(store, institute_obj, case_obj)
    return dict(institute=institute_obj, case=case_obj, format='html', **data)


@cases_bp.route('/<institute_id>/<case_name>/pdf_report', methods=['GET'])
def pdf_case_report(institute_id, case_name):
    """Download a pdf report for a case"""

    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    data = controllers.case_report_content(store, institute_obj, case_obj)

    # add coverage report on the bottom of this report
    if current_app.config.get('SQLALCHEMY_DATABASE_URI'):
        data['coverage_report'] = controllers.coverage_report_contents(store, institute_obj, case_obj, request.url_root)

    # workaround to be able to print the case pedigree to pdf
    if case_obj.get('madeline_info') is not None:
        with open(os.path.join(cases_bp.static_folder, 'madeline.svg'), 'w') as temp_madeline:
            temp_madeline.write(case_obj['madeline_info'])

    html_report = render_template('cases/case_report.html', institute=institute_obj, case=case_obj, format='pdf', **data)
    return render_pdf(HTML(string=html_report), download_filename=case_obj['display_name']+'_'+datetime.datetime.now().strftime("%Y-%m-%d")+'_scout.pdf')


@cases_bp.route('/<institute_id>/<case_name>/mt_report', methods=['GET'])
def mt_report(institute_id, case_name):
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)

    # create a temp folder to write excel files into
    temp_excel_dir = os.path.join(cases_bp.static_folder, '_'.join([case_name, 'mt_reports']))
    os.makedirs(temp_excel_dir, exist_ok=True)

    # create mt excel files, one for each sample
    n_files = controllers.mt_excel_files(store, case_obj, temp_excel_dir)

    if n_files:
        today = datetime.datetime.now().strftime('%Y-%m-%d')
        # zip the files on the fly and serve the archive to the user
        data = io.BytesIO()
        with zipfile.ZipFile(data, mode='w') as z:
            for f_name in pathlib.Path(temp_excel_dir).iterdir():
                zipfile.ZipFile
                z.write(f_name, os.path.basename(f_name))
        data.seek(0)

        # remove temp folder with excel files in it
        shutil.rmtree(temp_excel_dir)

        return send_file(
            data,
            mimetype='application/zip',
            as_attachment=True,
            attachment_filename='_'.join(['scout', case_name, 'MT_report', today])+'.zip'
        )
    else:
        flash('No MT report excel file could be exported for this sample', 'warning')
        return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/diagnose', methods=['POST'])
def case_diagnosis(institute_id, case_name):
    """Add or remove a diagnosis for a case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    link = url_for('.case', institute_id=institute_id, case_name=case_name)
    level = 'phenotype' if 'phenotype' in request.form else 'gene'
    omim_id = request.form['omim_id']
    remove = True if request.args.get('remove') == 'yes' else False
    store.diagnose(institute_obj, case_obj, user_obj, link, level=level,
                   omim_id=omim_id, remove=remove)
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/phenotypes', methods=['POST'])
@cases_bp.route('/<institute_id>/<case_name>/phenotypes/<phenotype_id>',
                methods=['POST'])
def phenotypes(institute_id, case_name, phenotype_id=None):
    """Handle phenotypes."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    case_url = url_for('.case', institute_id=institute_id, case_name=case_name)
    is_group = request.args.get('is_group') == 'yes'
    user_obj = store.user(current_user.email)

    if phenotype_id:
        # DELETE a phenotype item/group from case
        store.remove_phenotype(institute_obj, case_obj, user_obj, case_url,
                               phenotype_id, is_group=is_group)
    else:
        try:
            # add a new phenotype item/group to the case
            phenotype_term = request.form['hpo_term']
            if phenotype_term.startswith('HP:') or len(phenotype_term) == 7:
                hpo_term = phenotype_term.split(' | ', 1)[0]
                store.add_phenotype(institute_obj, case_obj, user_obj, case_url,
                                    hpo_term=hpo_term, is_group=is_group)
            else:
                # assume omim id
                store.add_phenotype(institute_obj, case_obj, user_obj, case_url,
                                    omim_term=phenotype_term)
        except ValueError:
            return abort(400, ("unable to add phenotype: {}".format(phenotype_term)))
    return redirect(case_url)


@cases_bp.route('/<institute_id>/<case_name>/phenotypes/actions', methods=['POST'])
def phenotypes_actions(institute_id, case_name):
    """Perform actions on multiple phenotypes."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    case_url = url_for('.case', institute_id=institute_id, case_name=case_name)
    action = request.form['action']
    hpo_ids = request.form.getlist('hpo_id')
    user_obj = store.user(current_user.email)

    if action == 'DELETE':
        for hpo_id in hpo_ids:
            # DELETE a phenotype from the list
            store.remove_phenotype(institute_obj, case_obj, user_obj, case_url, hpo_id)
    elif action == 'PHENOMIZER':
        if len(hpo_ids) == 0:
            hpo_ids = [term['phenotype_id'] for term in case_obj.get('phenotype_terms', [])]

        username = current_app.config['PHENOMIZER_USERNAME']
        password = current_app.config['PHENOMIZER_PASSWORD']
        diseases = controllers.hpo_diseases(username, password, hpo_ids)
        return render_template('cases/diseases.html', diseases=diseases,
                               institute=institute_obj, case=case_obj)

    elif action == 'GENES':
        hgnc_symbols = set()
        for raw_symbols in request.form.getlist('genes'):
            # avoid empty lists
            if raw_symbols:
                hgnc_symbols.update(raw_symbol.split(' ', 1)[0] for raw_symbol in
                                    raw_symbols.split('|'))
        store.update_dynamic_gene_list(case_obj, hgnc_symbols=hgnc_symbols)

    elif action == 'GENERATE':
        if len(hpo_ids) == 0:
            hpo_ids = [term['phenotype_id'] for term in case_obj.get('phenotype_terms', [])]
        results = store.generate_hpo_gene_list(*hpo_ids)
        # determine how many HPO terms each gene must match
        hpo_count = int(request.form.get('min_match') or 1)
        hgnc_ids = [result[0] for result in results if result[1] >= hpo_count]
        store.update_dynamic_gene_list(case_obj, hgnc_ids=hgnc_ids, phenotype_ids=hpo_ids)

    return redirect(case_url)


@cases_bp.route('/<institute_id>/<case_name>/events', methods=['POST'])
@cases_bp.route('/<institute_id>/<case_name>/events/<event_id>', methods=['POST'])
def events(institute_id, case_name, event_id=None):
    """Handle events."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    link = request.form.get('link')
    content = request.form.get('content')
    variant_id = request.args.get('variant_id')
    user_obj = store.user(current_user.email)

    if event_id:
        # delete the event
        store.delete_event(event_id)
    else:
        if variant_id:
            # create a variant comment
            variant_obj = store.variant(variant_id)
            level = request.form.get('level', 'specific')
            store.comment(institute_obj, case_obj, user_obj, link,
                          variant=variant_obj, content=content, comment_level=level)
        else:
            # create a case comment
            store.comment(institute_obj, case_obj, user_obj, link, content=content)

    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/status', methods=['POST'])
def status(institute_id, case_name):
    """Update status of a specific case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)

    status = request.form.get('status', case_obj['status'])
    link = url_for('.case', institute_id=institute_id, case_name=case_name)

    if status == 'archive':
        store.archive_case(institute_obj, case_obj, user_obj, status, link)
    else:
        store.update_status(institute_obj, case_obj, user_obj, status, link)

    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/assign', methods=['POST'])
@cases_bp.route('/<institute_id>/<case_name>/<user_id>/assign', methods=['POST'])
def assign(institute_id, case_name, user_id=None):
    """Assign and unassign a user from a case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    link = url_for('.case', institute_id=institute_id, case_name=case_name)
    if user_id:
        user_obj = store.user(user_id)
    else:
        user_obj = store.user(current_user.email)
    if request.form.get('action') == 'DELETE':
        store.unassign(institute_obj, case_obj, user_obj, link)
    else:
        store.assign(institute_obj, case_obj, user_obj, link)
    return redirect(request.referrer)


@cases_bp.route('/api/v1/hpo-terms')
def hpoterms():
    """Search for HPO terms."""
    query = request.args.get('query')
    if query is None:
        return abort(500)
    terms = sorted(store.hpo_terms(query=query), key=itemgetter('hpo_number'))
    json_terms = [
        {'name': '{} | {}'.format(term['_id'], term['description']),
         'id': term['_id']
        } for term in terms[:7]]

    return jsonify(json_terms)


@cases_bp.route('/<institute_id>/<case_name>/<variant_id>/pin', methods=['POST'])
def pin_variant(institute_id, case_name, variant_id):
    """Pin and unpin variants to/from the list of suspects."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    variant_obj = store.variant(variant_id)
    user_obj = store.user(current_user.email)
    link = url_for('variants.variant', institute_id=institute_id, case_name=case_name,
                   variant_id=variant_id)
    if request.form['action'] == 'ADD':
        store.pin_variant(institute_obj, case_obj, user_obj, link, variant_obj)
    elif request.form['action'] == 'DELETE':
        store.unpin_variant(institute_obj, case_obj, user_obj, link, variant_obj)
    return redirect(request.referrer or link)


@cases_bp.route('/<institute_id>/<case_name>/<variant_id>/validate', methods=['POST'])
def mark_validation(institute_id, case_name, variant_id):
    """Mark a variant as sanger validated."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    variant_obj = store.variant(variant_id)
    user_obj = store.user(current_user.email)
    validate_type = request.form['type'] or None
    link = url_for('variants.variant', institute_id=institute_id, case_name=case_name,
                   variant_id=variant_id)
    store.validate(institute_obj, case_obj, user_obj, link, variant_obj, validate_type)
    return redirect(request.referrer or link)


@cases_bp.route('/<institute_id>/<case_name>/<variant_id>/causative', methods=['POST'])
def mark_causative(institute_id, case_name, variant_id):
    """Mark a variant as confirmed causative."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    variant_obj = store.variant(variant_id)
    user_obj = store.user(current_user.email)
    link = url_for('variants.variant', institute_id=institute_id, case_name=case_name,
                   variant_id=variant_id)
    if request.form['action'] == 'ADD':
        store.mark_causative(institute_obj, case_obj, user_obj, link, variant_obj)
    elif request.form['action'] == 'DELETE':
        store.unmark_causative(institute_obj, case_obj, user_obj, link, variant_obj)

    # send the user back to the case that was marked as solved
    case_url = url_for('.case', institute_id=institute_id, case_name=case_name)
    return redirect(case_url)

@cases_bp.route('/<institute_id>/<case_name>/check-case', methods=['POST'])
def check_case(institute_id, case_name):
    """Mark a case that is has been checked.
       This means to set case['needs_check'] to False
    """
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    store.case_collection.find_one_and_update({'_id':case_obj['_id']}, {'$set': {'needs_check': False}})
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/delivery-report')
def delivery_report(institute_id, case_name):
    """Display delivery report."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    if case_obj.get('delivery_report') is None:
        return abort(404)

    date_str = request.args.get('date')
    if date_str:
        delivery_report = None
        analysis_date = parse_date(date_str)
        for analysis_data in case_obj['analyses']:
            if analysis_data['date'] == analysis_date:
                delivery_report = analysis_data['delivery_report']
        if delivery_report is None:
            return abort(404)
    else:
        delivery_report = case_obj['delivery_report']

    out_dir = os.path.dirname(delivery_report)
    filename = os.path.basename(delivery_report)
    return send_from_directory(out_dir, filename)

@cases_bp.route('/<institute_id>/<case_name>/share', methods=['POST'])
def share(institute_id, case_name):
    """Share a case with a different institute."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    collaborator_id = request.form['collaborator']
    revoke_access = 'revoke' in request.form
    link = url_for('.case', institute_id=institute_id, case_name=case_name)

    if revoke_access:
        store.unshare(institute_obj, case_obj, collaborator_id, user_obj, link)
    else:
        store.share(institute_obj, case_obj, collaborator_id, user_obj, link)

    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/rerun', methods=['POST'])
def rerun(institute_id, case_name):
    """Request a case to be rerun."""
    sender = current_app.config['MAIL_USERNAME']
    recipient = current_app.config['TICKET_SYSTEM_EMAIL']
    controllers.rerun(store, mail, current_user, institute_id, case_name, sender,
                      recipient)
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/research', methods=['POST'])
def research(institute_id, case_name):
    """Open the research list for a case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    link = url_for('.case', institute_id=institute_id, case_name=case_name)
    store.open_research(institute_obj, case_obj, user_obj, link)
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/cohorts', methods=['POST'])
def cohorts(institute_id, case_name):
    """Add/remove institute tags."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    link = url_for('.case', institute_id=institute_id, case_name=case_name)
    cohort_tag = request.form['cohort_tag']
    if request.args.get('remove') == 'yes':
        store.remove_cohort(institute_obj, case_obj, user_obj, link, cohort_tag)
    else:
        store.add_cohort(institute_obj, case_obj, user_obj, link, cohort_tag)
    return redirect(request.referrer)


@cases_bp.route('/<institute_id>/<case_name>/default-panels', methods=['POST'])
def default_panels(institute_id, case_name):
    """Update default panels for a case."""
    panel_ids = request.form.getlist('panel_ids')
    controllers.update_default_panels(store, current_user, institute_id, case_name, panel_ids)
    return redirect(request.referrer)

@cases_bp.route('/<institute_id>/<case_name>/<individual_id>/cgh')
def vcf2cytosure(institute_id, case_name, individual_id):
    """Download vcf2cytosure file for individual."""

    (display_name, vcf2cytosure) = controllers.vcf2cytosure(store,
        institute_id, case_name, individual_id)

    outdir = os.path.abspath(os.path.dirname(vcf2cytosure))
    filename = os.path.basename(vcf2cytosure)

    log.debug("Attempt to deliver file {0} from dir {1}".format(filename, outdir))

    attachment_filename = display_name + ".vcf2cytosure.cgh"

    return send_from_directory(outdir, filename,
        attachment_filename=attachment_filename,
        as_attachment=True)

@cases_bp.route('/<institute_id>/<case_name>/multiqc')
def multiqc(institute_id, case_name):
    """Load multiqc report for the case."""
    data = controllers.multiqc(store, institute_id, case_name)
    if data['case'].get('multiqc') is None:
        return abort(404)
    out_dir = os.path.abspath(os.path.dirname(data['case']['multiqc']))
    filename = os.path.basename(data['case']['multiqc'])
    return send_from_directory(out_dir, filename)



import os
import click
import logging
import datetime

from bson.json_util import dumps
from xlsxwriter import Workbook

from scout.export.variant import export_variants, export_verified_variants
from .utils import json_option

from scout.constants.variants_export import VCF_HEADER, VERIFIED_VARIANTS_HEADER

LOG = logging.getLogger(__name__)


@click.command('verified', short_help='Export validated variants')
@click.option('-c', '--collaborator',
        help="Specify what collaborator to export variants from. Defaults to cust000",
)
@click.option('--outpath',
              help='Path to output file'
)
@click.option('--test',
              help='Use this flag to test the function',
              is_flag=True
)
@click.pass_context
def verified(context, collaborator, test, outpath=None):
    """Export variants which have been verified for an institute
        and write them to an excel file.

    Args:
        collaborator(str): institute id
        test(bool): True if the function is called for testing purposes
        outpath(str): path to output file

    Returns:
        written_files(int): number of written or simulated files
    """
    written_files = 0
    collaborator = collaborator or 'cust000'
    LOG.info('Exporting verified variants for cust {}'.format(collaborator))

    adapter = context.obj['adapter']
    verified_vars = adapter.verified(institute_id=collaborator)
    LOG.info('FOUND {} verified variants for institute {}'.format(len(verified_vars), collaborator))


    if not verified_vars:
        LOG.warning('There are no verified variants for institute {} in database!'.format(collaborator))
        return None

    document_lines = export_verified_variants(verified_vars)

    today = datetime.datetime.now().strftime('%Y-%m-%d')
    document_name = '.'.join(['verified_variants', collaborator, today]) + '.xlsx'

    # If this was a test and lines are created return success
    if test and document_lines:
        written_files +=1
        LOG.info('Success. Verified variants file contains {} lines'.format(len(document_lines)))
        return written_files

    # create workbook and new sheet
    # set up outfolder
    if not outpath:
        outpath = str(os.getcwd())
    workbook = Workbook(os.path.join(outpath,document_name))
    Report_Sheet = workbook.add_worksheet()

    # Write the column header
    row = 0
    for col,field in enumerate(VERIFIED_VARIANTS_HEADER):
        Report_Sheet.write(row,col,field)

    # Write variant lines, after header (start at line 1)
    for row, line in enumerate(document_lines,1): # each line becomes a row in the document
        for col, field in enumerate(line): # each field in line becomes a cell
            Report_Sheet.write(row,col,field)
    workbook.close()

    if os.path.exists(os.path.join(outpath,document_name)):
        LOG.info('Success. Verified variants file of {} lines was written to disk'. format(len(document_lines)))
        written_files += 1

    return written_files


@click.command('variants', short_help='Export variants')
@click.option('-c', '--collaborator',
        help="Specify what collaborator to export variants from. Defaults to cust000",
)
@click.option('-d', '--document-id',
        help="Search for a specific variant",
)
@click.option('--case-id',
        help="Find causative variants for case",
)
@json_option
@click.pass_context
def variants(context, collaborator, document_id, case_id, json):
    """Export causatives for a collaborator in .vcf format"""
    LOG.info("Running scout export variants")
    adapter = context.obj['adapter']
    collaborator = collaborator or 'cust000'

    variants = export_variants(
        adapter,
        collaborator,
        document_id=document_id,
        case_id=case_id
    )

    if json:
        click.echo(dumps([var for var in variants]))
        return

    vcf_header = VCF_HEADER

    #If case_id is given, print more complete vcf entries, with INFO,
    #and genotypes
    if case_id:
        vcf_header[-1] = vcf_header[-1] + "\tFORMAT"
        case_obj = adapter.case(case_id=case_id)
        for individual in case_obj['individuals']:
            vcf_header[-1] = vcf_header[-1] + "\t" + individual['individual_id']

    #print header
    for line in vcf_header:
        click.echo(line)

    for variant_obj in variants:
        variant_string = get_vcf_entry(variant_obj, case_id=case_id)
        click.echo(variant_string)


def get_vcf_entry(variant_obj, case_id=None):
    """
        Get vcf entry from variant object

        Args:
            variant_obj(dict)
        Returns:
            variant_string(str): string representing variant in vcf format
    """
    if variant_obj['category'] == 'snv':
        var_type = 'TYPE'
    else:
        var_type = 'SVTYPE'

    info_field = ';'.join(
            [
                'END='+str(variant_obj['end']),
                var_type+'='+variant_obj['sub_category'].upper()
            ]
        )

    variant_string = "{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}".format(
        variant_obj['chromosome'],
        variant_obj['position'],
        variant_obj['dbsnp_id'],
        variant_obj['reference'],
        variant_obj['alternative'],
        variant_obj['quality'],
        ';'.join(variant_obj['filters']),
        info_field
    )

    if case_id:
        variant_string += "\tGT"
        for sample in variant_obj['samples']:
            variant_string += "\t" + sample['genotype_call']

    return variant_string



from scout.constants import (PAR_COORDINATES, CHR_PATTERN)

def is_par(chromosome, position, build='37'):
    """Check if a variant is in the Pseudo Autosomal Region or not
    
    Args:
        chromosome(str)
        position(int)
        build(str): The genome build
    
    Returns:
        bool
    """
    chrom_match = CHR_PATTERN.match(chromosome)
    chrom = chrom_match.group(2)
    
    # PAR regions are only on X and Y
    if not chrom in ['X','Y']:
        return False
    # Check if variant is in first PAR region
    if PAR_COORDINATES[build][chrom].search(position):
        return True
    
    return False

def check_coordinates(chromosome, pos, coordinates):
    """Check if the variant is in the interval given by the coordinates

        Args:
            chromosome(str): Variant chromosome
            pos(int): Variant position
            coordinates(dict): Dictionary with the region of interest
    """
    chrom_match = CHR_PATTERN.match(chromosome)
    chrom = chrom_match.group(2)

    if chrom != coordinates['chrom']:
        return False
    
    if (pos >= coordinates['start'] and pos <= coordinates['end']):
        return True

    return False



"""
commands/utils.py

Helper functions for cli functions

"""

import click

from scout.constants import BUILDS

def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()

builds_option = click.option('-b', '--build',
    default='37',
    show_default=True,
    type=click.Choice(BUILDS),
)



# -*- coding: utf-8 -*-

from scout.export.variant import export_mt_variants, export_verified_variants
from scout.constants.variants_export import MT_EXPORT_HEADER, VERIFIED_VARIANTS_HEADER

def test_export_mt_variants(case_obj, real_populated_database):
    """Test the function that creates lines with MT variants to be exported"""
    adapter = real_populated_database
    case_id = case_obj['_id']
    assert case_id

    # load MT variants for this case
    nr_loaded = adapter.load_variants(case_obj=case_obj,
                category='snv', chrom='MT', start=1, end=16500)
    assert nr_loaded > 0
    mt_variants = list(adapter.variant_collection.find({'chromosome':'MT'}))
    assert len(mt_variants) == nr_loaded # it's all MT variants, but double-checking it

    # Assert that there is at least one sample to create the excel file for
    samples = case_obj.get('individuals')
    assert samples

    # test function that exports variant lines
    for sample in samples:
        sample_lines = export_mt_variants(variants=mt_variants, sample_id=sample['individual_id'])

        # check that rows to write to excel corespond to number of variants
        assert len(sample_lines) == len(mt_variants)
        # check that cols to write to excel corespond to fields of excel header
        assert len(sample_lines[0]) == len(MT_EXPORT_HEADER)


def test_export_verified_variants(case_obj, real_populated_database, variant_objs):
    """Test the function that creates lines with verified variants to be exported"""

    adapter = real_populated_database
    case_id = case_obj['_id']

    # load variants to populated_database
    assert adapter.variants(case_id=case_id, nr_of_variants=-1).count() == 0
    nr_loaded = adapter.load_variants(case_obj=case_obj)
    assert nr_loaded > 0

    valid_status = ['False positive', 'True positive', 'False positive'] # for 3 vars

    test_vars = list(adapter.variant_collection.find().limit(3))
    assert len(test_vars) == 3

    # Make sure that no variant is set as validated:
    assert adapter.variant_collection.find({'validation':{'$exists':True}}).count()==0

    # Set test variant as validated
    for i in range(3):
        # Set validation status of a variant
        adapter.variant_collection.find_one_and_update(
            {'_id' : test_vars[i]['_id'] },
            {'$set' : {'validation' : valid_status[i] } }
         )

        # insert validation events
        adapter.event_collection.insert_one({
            'verb' : 'validate',
            'institute' : test_vars[i]['institute'],
            'variant_id' : test_vars[i]['variant_id'],
            'case' : case_id
         })

    # There should be 3 validated variants now
    assert adapter.variant_collection.find({'validation':{'$exists':True}}).count()==3

    # Call function to get aggregated data (variant + case data):
    cust = case_obj['owner']
    aggregated_vars = adapter.verified(cust)
    assert len(aggregated_vars) == 3 # same number of variants is returned

    # Call function that creates document lines
    document_lines = export_verified_variants(aggregated_vars)
    assert len(document_lines) == 3 # one line per variant should be returned

    for line in document_lines:
        # Make sure that document cells will be the same as in document header
        assert len(line) == len(VERIFIED_VARIANTS_HEADER)



import os
import logging

from pprint import pprint as pp

import click

from scout.utils.requests import (fetch_hgnc, fetch_mim_files, fetch_exac_constraint, 
fetch_ensembl_genes, fetch_ensembl_transcripts, fetch_hpo_files, fetch_hpo_genes, fetch_hpo_terms,)
from scout.parse.hgnc import parse_hgnc_line
from scout.parse.omim import parse_genemap2, parse_mim2gene
from scout.parse.exac import parse_exac_genes
from scout.parse.ensembl import (parse_ensembl_gene_request, parse_ensembl_transcript_request)

from . import (hgnc_reduced_path, genemap2_reduced_path, mim2gene_reduced_path, exac_reduced_path, 
genes37_reduced_path, genes38_reduced_path, transcripts37_reduced_path, transcripts38_reduced_path,
hpogenes_reduced_path, hpoterms_reduced_path, hpo_phenotype_to_terms_reduced_path)


LOG = logging.getLogger(__name__)

def get_reduced_hpo_terms(hpo_terms):
    """Return a reduced version of the hpo terms
    
    Args:
        hpo_terms(set(str)): Set of choosen terms that should be included

    Yields:
        hpo_line: A line with hpo information
    """
    hpo_lines = fetch_hpo_terms()
    
    begining = True
    
    term_lines = []
    # We want to keep the header lines
    keep = True
    
    nr_terms = 0
    nr_kept = 0

    for line in hpo_lines:
        
        # When we encounter a new term we yield all lines of the previous term
        if line.startswith('[Term]'):
            nr_terms += 1
            if keep:
                nr_kept += 1
                for hpo_line in term_lines:
                    yield hpo_line

            keep = False
            term_lines = []
        
        elif line.startswith('id'):
            hpo_id = line[4:]
            if hpo_id in hpo_terms:
                keep = True
        
        term_lines.append(line)
        

    if keep:
        for hpo_line in term_lines:
            yield hpo_line
    
    LOG.info("Nr of terms in file %s", nr_terms)
    LOG.info("Nr of terms kept: %s", nr_kept)
        

def remove_file(path):
    """Check if a file exists and remove it if so
    
    Args:
        path(str)
    """
    LOG.info("Removing file %s", path)
    try:
        os.remove(path)
        LOG.info("File %s removed", path)
    except OSError as err:
        LOG.info("File %s does not exists", path)
    return

def generate_hgnc(genes):
    """Generate lines from a file with reduced hgnc information
    
    Args:
        genes(dict): A dictionary with hgnc_id as key and hgnc_symbol as value
        outpath(str): Defaults to hgnc_reduced_path
    
    Yields:
        print_line(str): Lines from the reduced file
    """
    LOG.info("Generating new hgnc reduced file")
    # fetch the latest hgnc file here
    hgnc_gene_lines = fetch_hgnc() 

    header = None
    genes_found = 0
    
    # Loop over all hgnc gene lines
    for i,line in enumerate(hgnc_gene_lines):
        
        line = line.rstrip()
        # Skip lines that are empty
        if not len(line) > 0:
            continue
        # If we are reading the header, print it
        if i == 0:
            header = line.split('\t')
            yield line
            continue

        # Parse the hgnc gene line
        gene = parse_hgnc_line(line, header)
        if not gene:
            continue
        hgnc_id = int(gene['hgnc_id'])
        # Check if the gene is in the reduced
        if hgnc_id in genes:
            genes_found += 1
            yield line
    
    LOG.info("Number of genes printed to file: %s", genes_found)

def generate_genemap2(genes, api_key):
    """Generate a reduced file with omim genemap2 information
    
    Args:
        genes(dict): A dictionary with hgnc_symbol as key and hgnc_id as value
        api_key(str)

    Yields:
        print_line(str): Lines from the reduced file
    """
    
    mim_files = fetch_mim_files(api_key, genemap2=True)
    genemap2_lines = mim_files['genemap2']
    
    # Yield the header lines
    for line in genemap2_lines:
        if line.startswith('#'):
            yield line
        else:
            break
    
    for gene_info in parse_genemap2(genemap2_lines):
        hgnc_symbol = gene_info.get('hgnc_symbol')
        if not hgnc_symbol:
            continue
        if hgnc_symbol in genes:
            yield gene_info['raw']

def generate_mim2genes(genes, api_key):
    """Generate a reduced file with omim mim2gene information
    
    Args:
        genes(dict): A dictionary with hgnc_symbol as key and hgnc_id as value
        api_key(str)

    Yields:
        print_line(str): Lines from the reduced file
    """
    
    mim_files = fetch_mim_files(api_key, mim2genes=True)
    mim2gene_lines = mim_files['mim2genes']
    
    for line in mim2gene_lines:
        if line.startswith('#'):
            yield line
        else:
            break
    
    for gene_info in parse_mim2gene(mim2gene_lines):
        hgnc_symbol = gene_info.get('hgnc_symbol')
        if not hgnc_symbol:
            continue
        if hgnc_symbol in genes:
            yield gene_info['raw']

def generate_exac_genes(genes):
    """Generate a reduced file with omim mim2gene information
    
    Args:
        genes(dict): A dictionary with hgnc_symbol as key and hgnc_id as value
        outpath(str)

    Yields:
        print_line(str): Lines from the reduced file
    """
    exac_lines = fetch_exac_constraint()

    yield(exac_lines[0])
    
    for gene_info in parse_exac_genes(exac_lines):
        hgnc_symbol = gene_info.get('hgnc_symbol')
        if not hgnc_symbol:
            continue
        if hgnc_symbol in genes:
            yield gene_info['raw']

def generate_ensembl_genes(genes, silent=False, build=None):
    """Generate gene lines from a build

    Args:
        genes(dict): A dictionary with hgnc_id as key and hgnc_symbol as value
        silent(bool): If genes should be written to file or not
        build(str): What build to use. Defaults to 37
    
    Yields:
        print_line(str):  Lines from the reduced file
    """
    build = build or '37'

    ensembl_header = ['Chromosome/scaffold name', 'Gene start (bp)', 'Gene end (bp)',
                      'Gene stable ID', 'HGNC symbol', 'HGNC ID']

    yield '\t'.join(ensembl_header)
    
    ensembl_genes = fetch_ensembl_genes(build=build)

    nr_genes = 0

    # This function will yield dictionaries with ensemble info
    for gene_info in parse_ensembl_gene_request(ensembl_genes):
        hgnc_id = gene_info.get('hgnc_id')
        if not hgnc_id:
            continue
        if hgnc_id in genes:
            print_line = [
                gene_info['chrom'],
                str(gene_info['gene_start']),
                str(gene_info['gene_end']),
                gene_info['ensembl_gene_id'],
                gene_info['hgnc_symbol'],
                str(gene_info['hgnc_id']),
            ]
            yield '\t'.join(print_line)
            nr_genes += 1
    
    LOG.info("Nr genes collected for build %s: %s", build,nr_genes)

def generate_ensembl_transcripts(ensembl_genes, build):
    """Generate a file with reduced ensembl gene information
    
    Args:
        genes(dict): A dictionary with ensembl_id as key and hgnc_id as value
        silent(bool): If genes should be written to file or not
        build(str): What build to use. Defaults to 37
    
    Yields:
        print_line(str):  Lines from the reduced file
    
    """
    build = build or '37'
    
    ensembl_transcripts = fetch_ensembl_transcripts(build=build)
        
    ensembl_header = ['Chromosome/scaffold name', 'Gene stable ID', 
                   'Transcript stable ID', 'Transcript start (bp)', 
                   'Transcript end (bp)', 'RefSeq mRNA ID',
                   'RefSeq mRNA predicted ID', 'RefSeq ncRNA ID']
        
        
    yield '\t'.join(ensembl_header)
        
    for tx_info in parse_ensembl_transcript_request(ensembl_transcripts):
        ens_gene_id = tx_info['ensembl_gene_id']
        if ens_gene_id in ensembl_genes:
            print_line = [
                tx_info['chrom'],
                tx_info['ensembl_gene_id'],
                tx_info['ensembl_transcript_id'],
                str(tx_info['transcript_start']),
                str(tx_info['transcript_end']),
                tx_info['refseq_mrna'] or '',
                tx_info['refseq_mrna_predicted'] or '',
                tx_info['refseq_ncrna'] or '',
            ]
            yield '\t'.join(print_line)

def generate_hpo_genes(genes):
    """Generate the lines from a reduced hpo genes file
    
    Args:
        genes(dict): A map from hgnc_symbol to hgnc_id
    
    Yields:
        line(str): Lines from hpo with connection to genes
    """
    hpo_lines = fetch_hpo_genes()
    nr_terms = 0
    
    for i,line in enumerate(hpo_lines):
        line = line.rstrip()
        if not len(line) > 1:
            continue
        #Header line
        if i == 0:
            yield line
            continue

        splitted_line = line.split('\t')
        hgnc_symbol = splitted_line[1]
        
        if hgnc_symbol in genes:
            nr_terms
            yield line

def generate_hpo_terms(genes):
    """Generate the lines from a reduced hpo terms file
    
    Args:
        genes(dict): A map from hgnc_symbol to hgnc_id
    
    Yields:
        line(str): Lines from hpo with connection to genes
    """
    hpo_lines = fetch_hpo_genes()
    nr_terms = 0
    
    for i,line in enumerate(hpo_lines):
        line = line.rstrip()
        if not len(line) > 1:
            continue
        #Header line
        if i == 0:
            yield line
            continue

        splitted_line = line.split('\t')
        hgnc_symbol = splitted_line[1]
        
        if hgnc_symbol in genes:
            nr_terms
            yield line


def generate_hpo_files(genes):
    """Generate files with hpo reduced information"""
    hpo_files = fetch_hpo_files(hpogenes=True, hpoterms=True, phenotype_to_terms=True, hpodisease=False)
    
    file_names = {
        'hpogenes': hpogenes_reduced_path,
        'hpoterms': hpoterms_reduced_path,
        'phenotype_to_terms': hpo_phenotype_to_terms_reduced_path
    }
    
    for name in file_names:
        hpo_lines = hpo_files[name]
        out_path = file_names[name]
        outfile = open(out_path, 'w')
        LOG.info('Writing file %s', out_path)
        
        for i,line in enumerate(hpo_lines):
            line = line.rstrip()
            if not len(line) > 1:
                continue
            if i == 0:#Header line
                outfile.write(line+'\n')
                continue
            splitted_line = line.split('\t')
            if name == 'hpogenes':
                hgnc_symbol = splitted_line[1]
            elif name == 'hpoterms':
                hgnc_symbol = splitted_line[3]
            elif name == 'phenotype_to_terms':
                hgnc_symbol = splitted_line[1]
            
            if hgnc_symbol in genes:
                outfile.write(line+'\n')
        LOG.info("File ready")



from scout.parse.hpo import (parse_hpo_genes, parse_hpo_gene)

def test_parse_hpo_gene_line(hpo_genes_handle):
    header = next(hpo_genes_handle)
    first_gene = next(hpo_genes_handle)

    gene_info = parse_hpo_gene(first_gene)

    assert gene_info['hgnc_symbol'] == first_gene.split('\t')[1]

def test_parse_hpo_genes(hpo_genes_handle):
    header = next(hpo_genes_handle)

    genes = parse_hpo_genes(hpo_genes_handle)
    for hgnc_symbol in genes:
        assert genes[hgnc_symbol]['hgnc_symbol']


# -*- coding= utf-8 -*-
"""

"main concept of MongoDB is embed whenever possible"
Ref= http=//stackoverflow.com/questions/4655610#comment5129510_4656431
"""
from datetime import datetime

class Institute(dict):
    """Represents a institute
    
    _id = str, # same as internal_id
    internal_id = str, # like 'cust000', required
    display_name = str, # like 'Clinical Genomics', required
    sanger_recipients = list, # list of email adressess

    created_at = datetime, # Defaults to now
    updated_at = datetime, # Defaults to now

    coverage_cutoff = int, # Defaults to  10
    frequency_cutoff = float, # Defaults to 0.01
    
    """
    def __init__(self, internal_id, display_name, sanger_recipients, created_at=None, 
                 updated_at=None, coverage_cutoff=None, frequency_cutoff=None):
        super(Institute, self).__init__()
        self['internal_id'] = internal_id
        self['_id'] = internal_id
        self['display_name'] = display_name
        self['sanger_recipients'] = sanger_recipients
        self['created_at'] = created_at or datetime.now()
        self['updated_at'] = updated_at or datetime.now()
        self['coverage_cutoff'] = coverage_cutoff or 10
        self['frequency_cutoff'] = frequency_cutoff or 0.01




import logging

import click

from bson.json_util import dumps

from scout.commands.utils import builds_option

from scout.export.gene import export_genes

from .utils import (build_option, json_option)

LOG = logging.getLogger(__name__)

@click.command('genes', short_help='Export genes')
@build_option
@json_option
@click.pass_context
def genes(context, build, json):
    """Export all genes from a build"""
    LOG.info("Running scout export genes")
    adapter = context.obj['adapter']

    result = adapter.all_genes(build=build)
    if json:
        click.echo(dumps(result))
        return
        
    gene_string = ("{0}\t{1}\t{2}\t{3}\t{4}")
    click.echo("#Chromosom\tStart\tEnd\tHgnc_id\tHgnc_symbol")
    for gene_obj in result:
        click.echo(gene_string.format(
            gene_obj['chromosome'],
            gene_obj['start'],
            gene_obj['end'],
            gene_obj['hgnc_id'],
            gene_obj['hgnc_symbol'],
        ))




# -*- coding: utf-8 -*-
import pytest
from pprint import pprint as pp
from scout.exceptions import (PedigreeError, ConfigError, IntegrityError)

from scout.build import build_case


def test_build_case(parsed_case, panel_database):
    adapter = panel_database

    # GIVEN a parsed case
    # WHEN bulding a case model
    case_obj = build_case(parsed_case, adapter)

    # THEN make sure it is built in the proper way
    assert case_obj['_id'] == parsed_case['case_id']
    assert case_obj['display_name'] == parsed_case['display_name']
    assert case_obj['owner'] == parsed_case['owner']
    assert case_obj['collaborators'] == parsed_case['collaborators']
    assert len(case_obj['individuals']) == len(parsed_case['individuals'])
    assert case_obj['synopsis'] == ''

    assert case_obj['status'] == 'inactive'
    assert case_obj['is_research'] == False
    assert case_obj['research_requested'] == False
    assert case_obj['rerun_requested'] == False

    assert case_obj['analysis_date'] == parsed_case['analysis_date']

    assert len(case_obj['dynamic_gene_list']) == 0

    assert case_obj['genome_build'] == parsed_case['genome_build']

    assert case_obj['rank_model_version'] == parsed_case['rank_model_version']
    assert case_obj['rank_score_threshold'] == parsed_case['rank_score_threshold']

    assert case_obj['madeline_info'] == parsed_case['madeline_info']

    assert case_obj['delivery_report'] == parsed_case['delivery_report']

    for vcf in case_obj['vcf_files']:
        assert vcf in parsed_case['vcf_files']

    # assert case_obj['diagnosis_phenotypes'] == []
    # assert case_obj['diagnosis_genes'] == []

    if (parsed_case['vcf_files'].get('vcf_sv') or parsed_case['vcf_files'].get('vcf_sv_research')):
        assert case_obj['has_svvariants'] == True
    else:
        assert case_obj['has_svvariants'] == False


def test_build_minimal_case(institute_database):
    # GIVEN a case without case id
    case_info = {
        'case_id': 'test-case',
        'owner': 'cust000'
    }
    # WHEN case is built
    case_obj = build_case(case_info, institute_database)
    # THEN assert that it worked
    assert case_obj['_id'] == case_info['case_id']


def test_build_case_no_case_id(adapter):
    # GIVEN a case without case id
    case_info = {}

    # WHEN case is built
    # THEN a PedigreeError should be raised
    with pytest.raises(Exception):
        build_case(case_info, adapter)


def test_build_case_no_display_name(institute_database):
    # GIVEN a case without case id
    case_info = {
        'case_id': 'test-case',
        'owner': 'cust000'
    }
    # WHEN case is built
    case_obj = build_case(case_info, institute_database)
    # THEN assert that display_name was set to case_id
    assert case_obj['display_name'] == case_info['case_id']


def test_build_case_no_owner(institute_database):
    # GIVEN a case where owner does not exist in the database
    case_info = {
        'case_id': 'test-case',
        'owner': 'cust001'
    }
    # WHEN case is built
    # THEN a IntegrityError should be raised since the owner has to exist in the database
    with pytest.raises(IntegrityError):
        case_obj = build_case(case_info, institute_database)


def test_build_case_non_existing_owner(institute_database):
    # GIVEN a case without owner
    # GIVEN a case where owner does not exist in the database
    case_info = {
        'case_id': 'test-case',
    }
    # WHEN case is built
    # THEN a ConfigError should be raised since a case has to have a owner
    with pytest.raises(ConfigError):
        case_obj = build_case(case_info, institute_database)

# def test_build_case_config(parsed_case):
#     case_obj = build_case(parsed_case)
#     print(case_obj.to_json())
#     assert False



from scout.constants import CONSERVATION
import numbers

def parse_conservations(variant):
    """Parse the conservation predictors

        Args:
            variant(dict): A variant dictionary

        Returns:
            conservations(dict): A dictionary with the conservations
    """
    conservations = {}

    conservations['gerp'] = parse_conservation(
                                            variant,
                                            'dbNSFP_GERP___RS'
                                        )
    conservations['phast'] = parse_conservation(
                                            variant,
                                            'dbNSFP_phastCons100way_vertebrate'
                                        )
    conservations['phylop'] = parse_conservation(
                                            variant,
                                            'dbNSFP_phyloP100way_vertebrate'
                                        )
    return conservations

def parse_conservation(variant, info_key):
    """Get the conservation prediction

        Args:
            variant(dict): A variant dictionary
            info_key(str)

        Returns:
            conservations(list): List of censervation terms
    """
    raw_score = variant.INFO.get(info_key)
    conservations = []

    if raw_score:
        if isinstance(raw_score, numbers.Number):
            raw_score = (raw_score,)

        for score in raw_score:
            if score >= CONSERVATION[info_key]['conserved_min']:
                conservations.append('Conserved')
            else:
                conservations.append('NotConserved')

    return conservations



# -*- coding: utf-8 -*-
import pytest
import logging
import datetime
from pprint import pprint as pp

from scout.exceptions import (IntegrityError)

logger = logging.getLogger(__name__)


def test_add_cases(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.case_collection.find().count() == 0

    # WHEN adding a new case to the database
    adapter._add_case(case_obj)

    # THEN it should be populated with the new case
    result = adapter.cases()
    assert result.count() == 1
    for case in result:
        assert case['owner'] == case_obj['owner']

    logger.info("All cases checked")


def test_add_existing_case(adapter,case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    adapter._add_case(case_obj)
    # WHEN adding a existing case to the database
    with pytest.raises(IntegrityError):
        # THEN it should raise integrity error
        adapter._add_case(case_obj)


def test_get_case(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    logger.info("Testing to get case")

    # WHEN retreiving an existing case from the database
    result = adapter.case(case_id=case_obj['_id'])
    # THEN we should get the correct case
    assert result['owner'] == case_obj['owner']


def test_get_cases(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    # WHEN retreiving an existing case from the database
    result = adapter.cases()
    # THEN we should get the correct case
    assert result.count() == 1


def test_get_cases_no_assignees(real_adapter, case_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    # WHEN retreiving an existing case from the database
    result = adapter.cases(name_query='john')
    # THEN we should get the correct case
    assert result.count() == 0


def test_get_cases_display_name(real_adapter, case_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    
    other_case = case_obj
    other_case['_id'] = 'other_case'
    other_case['display_name'] = 'other_case'
    adapter.case_collection.insert_one(other_case)
    
    # WHEN retreiving cases by partial display name
    result = adapter.cases(name_query='643')
    # THEN we should get the correct case
    assert result.count() == 1


def test_get_cases_existing_individual(real_adapter, case_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    # WHEN retreiving cases by partial individual name
    result = adapter.cases(name_query='NA1288')
    # THEN we should get the correct case
    assert result.count() == 1


def test_get_cases_assignees(real_adapter, case_obj, user_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    adapter.user_collection.insert_one(user_obj)
    
    user_obj = adapter.user_collection.find_one()
    case_obj['assignees'] = [user_obj['email']]
    adapter.case_collection.insert_one(case_obj)

    # WHEN retreiving cases by partial individual name
    result = adapter.cases(name_query='john')
    # THEN we should get the correct case
    assert result.count() == 1


def test_get_cases_non_existing_assignee(real_adapter, case_obj, user_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    adapter.user_collection.insert_one(user_obj)

    user_obj = adapter.user_collection.find_one()
    case_obj['assignees'] = [user_obj['email']]
    adapter.case_collection.insert_one(case_obj)

    # WHEN retreiving cases by partial individual name
    result = adapter.cases(name_query='damien')
    # THEN we should get the correct case
    assert result.count() == 0

def test_get_cases_causatives(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    # Add a causative
    case_obj['causatives'] = ['a variant']
    # Insert the case
    adapter.case_collection.insert_one(case_obj)

    # WHEN retreiving cases that have causatives
    result = adapter.cases(has_causatives=True)
    # THEN we should find one case
    assert result.count() == 1


def test_get_cases_causatives_no_causatives(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    # Insert a case without causatives
    adapter.case_collection.insert_one(case_obj)

    # WHEN retreiving all cases that have causatives 
    result = adapter.cases(has_causatives=True)
    # THEN we should get the correct case
    assert result.count() == 0

def test_get_cases_empty_causatives(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0

    # Add a empty list as causatives
    case_obj['causatives'] = []
    # Insert the case
    adapter.case_collection.insert_one(case_obj)

    # WHEN retreiving all cases that have causatives 
    result = adapter.cases(has_causatives=True)
    # THEN we should not find any cases
    assert result.count() == 0

def test_get_cases_non_existing_individual(real_adapter, case_obj):
    adapter = real_adapter
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    # WHEN retreiving cases by partial display name
    result = adapter.cases(name_query='hello')
    # THEN we should get the correct case
    assert result.count() == 0


def test_get_non_existing_case(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter._add_case(case_obj)
    logger.info("Testing to get case")

    # WHEN retreiving an non existing case from the database
    result = adapter.case(case_id='hello')
    # THEN we should get None back
    assert result is None


def test_delete_case(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    assert adapter.cases().count() == 1
    logger.info("Testing to delete case")

    # WHEN deleting a case from the database
    result = adapter.delete_case(case_id=case_obj['_id'])
    # THEN there should be no cases left in the database
    assert adapter.cases().count() == 0


def test_update_case_collaborators(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    assert adapter.cases().count() == 1
    assert len(adapter.case(case_obj['_id'])['collaborators']) == 1
    logger.info("Testing to update case")

    coll_1 = case_obj['collaborators'][0]
    coll_2 = 'test'
    coll_3 = 'test2'
    case_obj['collaborators'].append(coll_2)
    case_obj['collaborators'].append(coll_3)

    # WHEN updating a case with new collaborators
    res = adapter.update_case(case_obj)

    # THEN assert collaborator has been added
    assert len(res['collaborators']) == 3
    # THEN assert all collaborators where added
    assert set(res['collaborators']) == set([coll_1, coll_2, coll_3])


def test_update_case_individuals(adapter, case_obj):
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    assert adapter.cases().count() == 1
    logger.info("Testing to update case")

    res = adapter.case(case_obj['_id'])
    assert len(res['individuals']) == 3

    new_individuals = [{
        'individual_id': 'test',
        'display_name': 'test_name',
    }]
    case_obj['individuals'] = new_individuals
    # WHEN updating a case with new individuals
    res = adapter.update_case(case_obj)
    # THEN assert that 'individuals' has changed

    assert len(res['individuals']) == 1


def test_update_case_rerun_status(adapter, case_obj):
    case_obj['rerun_requested'] = True
    # GIVEN an empty database (no cases)
    assert adapter.cases().count() == 0
    adapter.case_collection.insert_one(case_obj)
    assert adapter.cases().count() == 1
    logger.info("Testing to update case")

    res = adapter.case(case_obj['_id'])
    assert res['rerun_requested'] is True

    # WHEN updating a case
    res = adapter.update_case(case_obj)

    # THEN assert that 'rerun_requested' is set to False
    assert res['rerun_requested'] is False



import logging

from pprint import pprint as pp

from scout.exceptions import IntegrityError

LOG = logging.getLogger(__name__)

def update_panel(adapter, panel_name, panel_version, new_version=None, new_date=None):
    """Update a gene panel in the database
    
    We need to update the actual gene panel and then all cases that refers to the panel.
    
    Args:
        adapter(scout.adapter.MongoAdapter)
        panel_name(str): Unique name for a gene panel
        panel_version(float)
        new_version(float)
        new_date(datetime.datetime)
    
    Returns:
        updated_panel(scout.models.GenePanel): The updated gene panel object
    """
    panel_obj = adapter.gene_panel(panel_name, panel_version)

    if not panel_obj:
        raise IntegrityError("Panel %s version %s does not exist" % (panel_name, panel_version))

    updated_panel = adapter.update_panel(panel_obj, new_version, new_date)
    
    panel_id = updated_panel['_id']

    # We need to alter the embedded panels in all affected cases
    update = {'$set': {}}
    if new_version:
        update['$set']['panels.$.version'] = updated_panel['version']
    if new_date:
        update['$set']['panels.$.updated_at'] = updated_panel['date']
    
    LOG.info('Updating affected cases with {0}'.format(update))
    
    query = {'panels': { '$elemMatch': {'panel_name': panel_name}}}
    adapter.case_collection.update_many(query, update)
    
    return updated_panel


import logging
import click

from scout.load.report import load_delivery_report

LOG = logging.getLogger(__name__)


@click.command()
@click.argument('case_id')
@click.argument('analysis_date')
@click.argument('report_path', type=click.Path(exists=True))
@click.argument('update', required=False)
@click.pass_context
def delivery_report(context, case_id, analysis_date, report_path,
                    update):
    """Add delivery report to an existing case."""

    adapter = context.obj['adapter']

    try:
        load_delivery_report(adapter, case_id, analysis_date,
                             report_path, update)
        LOG.info("saved report to case!")
    except Exception as e:
        LOG.error(e)
        context.abort()



import datetime

def build_evaluation(variant_specific, variant_id, user_id, user_name,
                     institute_id, case_id, classification, criteria):
    """Build a evaluation object ready to be inserted to database

    Args:
        variant_specific(str): md5 string for the specific variant
        variant_id(str): md5 string for the common variant
        user_id(str)
        user_name(str)
        institute_id(str)
        case_id(str)
        classification(str): The ACMG classification
        criteria(list(dict)): A list of dictionaries with ACMG criterias

    Returns:
        evaluation_obj(dict): Correctly formatted evaluation object

    """
    criteria = criteria or []
    evaluation_obj = dict(
        variant_specific = variant_specific,
        variant_id = variant_id,
        institute_id = institute_id,
        case_id = case_id,
        classification = classification,
        user_id = user_id,
        user_name = user_name,
        created_at = datetime.datetime.now(),
    )
    criteria_objs = []
    for info in criteria:
        criteria_obj = {}
        # This allways has to exist
        # We might want to check if the term is valid here...
        criteria_obj['term'] = info['term']
        if 'comment' in info:
            criteria_obj['comment'] = info['comment']
        if 'links' in info:
            criteria_obj['links'] = info['links']
        criteria_objs.append(criteria_obj)

    evaluation_obj['criteria'] = criteria_objs

    return evaluation_obj





import logging
import click

LOG = logging.getLogger(__name__)


@click.command('panels', short_help='Display gene panels')
@click.option('-i', '--institute',
              help='institute id'
              )
@click.pass_context
def panels(context, institute):
    """Show all gene panels in the database"""
    LOG.info("Running scout view panels")
    adapter = context.obj['adapter']

    panel_objs = adapter.gene_panels(institute_id=institute)
    if panel_objs.count() == 0:
        LOG.info("No panels found")
        context.abort()
    click.echo("#panel_name\tversion\tnr_genes\tdate")

    for panel_obj in panel_objs:
        click.echo("{0}\t{1}\t{2}\t{3}".format(
            panel_obj['panel_name'],
            str(panel_obj['version']),
            len(panel_obj['genes']),
            str(panel_obj['date'].strftime('%Y-%m-%d'))
        ))




import pytest

from scout.exceptions import IntegrityError

#########################################################
################### Hpo tests #######################
#########################################################

def test_add_hpo_term(adapter):
    ## GIVEN a empty adapter
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    ## WHEN loading a hpo term
    adapter.load_hpo_term(hpo_term)
    
    ## THEN assert that the term have been loaded
    assert len([term for term in adapter.hpo_terms()]) == 1

def test_add_hpo_term_twice(adapter):
    ## GIVEN a empty adapter
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    ## WHEN loading a hpo term
    adapter.load_hpo_term(hpo_term)
    with pytest.raises(IntegrityError):
        adapter.load_hpo_term(hpo_term)

def test_fetch_term(adapter):
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    ## WHEN fetching the hpo terms
    res = adapter.hpo_term(hpo_term['_id'])
    
    ## THEN assert the term was fetched
    assert res['_id'] == hpo_term['_id']

def test_fetch_non_existing_hpo_term(adapter):
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    ## WHEN fetching the hpo terms
    res = adapter.hpo_term('non existing')
    
    ## THEN assert resut is None
    assert res is None

def test_fetch_all_hpo_terms(adapter):
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    ## WHEN fetching the hpo terms
    res = [term for term in adapter.hpo_terms()]

    
    ## THEN assert the term was fetched
    assert len(res) == 1

def test_fetch_all_hpo_terms_query(real_adapter):
    adapter = real_adapter
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    hpo_term2 = dict(
        _id = 'HP2', # Same as hpo_id
       hpo_id = 'HP2', # Required
       description = 'Second term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term2)

    ## WHEN fetching the hpo terms with partial query
    res = [term for term in adapter.hpo_terms(query = '1')]
    
    ## THEN assert only one term was matched
    assert len(res) == 1

def test_fetch_all_hpo_terms_query_description(real_adapter):
    adapter = real_adapter
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    hpo_term2 = dict(
        _id = 'HP2', # Same as hpo_id
       hpo_id = 'HP2', # Required
       description = 'Second term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term2)

    ## WHEN fetching the hpo terms with partial query
    res = [term for term in adapter.hpo_terms(query = 'second')]
    
    ## THEN assert only one term was matched
    assert len(res) == 1
    for term in res:
        assert term['_id'] == 'HP2'

def test_fetch_all_hpo_terms_query_description_term(real_adapter):
    adapter = real_adapter
    ## GIVEN a adapter with one hpo term
    assert len([term for term in adapter.hpo_terms()]) == 0
    
    hpo_term = dict(
        _id = 'HP1', # Same as hpo_id
       hpo_id = 'HP1', # Required
       description = 'First term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term)

    hpo_term2 = dict(
        _id = 'HP2', # Same as hpo_id
       hpo_id = 'HP2', # Required
       description = 'Second term',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_hpo_term(hpo_term2)

    ## WHEN fetching the hpo terms with partial query
    res = adapter.hpo_terms(query = 'term')
    
    ## THEN assert only one term was matched
    assert len([term for term in res]) == 2


#########################################################
################### Disease tests #######################
#########################################################

def test_add_disease_term(adapter):
    ## GIVEN a empty adapter
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    ## WHEN loading a hpo term
    adapter.load_disease_term(disease_term)
    
    res = adapter.disease_terms()
    
    ## THEN assert that the term have been loaded
    assert len([term for term in res]) == 1

def test_add_disease_term(adapter):
    ## GIVEN a empty adapter
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    ## WHEN loading a hpo term twice
    adapter.load_disease_term(disease_term)
    
    ## THEN IntegrityError should be raised
    with pytest.raises(IntegrityError):
        adapter.load_disease_term(disease_term)

def test_fetch_disease_term(adapter):
    ## GIVEN a adapter loaded with one disease term
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_disease_term(disease_term)
    ## WHEN fetching a disease term
    res = adapter.disease_term(disease_term['_id'])
    
    ## THEN assert the correct term was fetched
    assert res['_id'] == disease_term['_id']

def test_fetch_disease_term_by_number(adapter):
    ## GIVEN a adapter loaded with one disease term
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_disease_term(disease_term)
    ## WHEN fetching a disease term
    res = adapter.disease_term(disease_term['disease_nr'])
    
    ## THEN assert the correct term was fetched
    assert res['_id'] == disease_term['_id']

def test_fetch_disease_term_by_hgnc_id(adapter):
    ## GIVEN a adapter loaded with one disease term
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_disease_term(disease_term)
    
    disease_term['_id'] = 'OMIM:2'
    disease_term['disease_id'] = 'OMIM:2'
    disease_term['disease_nr'] = '2'
    adapter.load_disease_term(disease_term)
    
    ## WHEN fetching a disease term
    res = adapter.disease_terms(hgnc_id=1)
    
    ## THEN assert the correct term was fetched
    assert len([term for term in res]) == 2

def test_fetch_disease_term_by_hgnc_id_again(adapter):
    ## GIVEN a adapter loaded with one disease term
    assert len([term for term in adapter.disease_terms()]) == 0
    
    disease_term = dict(
        _id = 'OMIM:1', 
       disease_id = 'OMIM:1',
       disease_nr = 1,
       source = 'OMIM',
       description = 'First disease',
       genes = [1], # List with integers that are hgnc_ids
    )
    adapter.load_disease_term(disease_term)
    
    disease_term['_id'] = 'OMIM:2'
    disease_term['disease_id'] = 'OMIM:2'
    disease_term['disease_nr'] = '2'
    disease_term['genes'] = [2]
    adapter.load_disease_term(disease_term)
    
    ## WHEN fetching a disease term
    res = adapter.disease_terms(hgnc_id=1)
    
    ## THEN assert the correct term was fetched
    assert len([term for term in res]) == 1



from scout.constants import CALLERS


def parse_callers(variant, category='snv'):
    """Parse how the different variant callers have performed

        Args:
            variant (cyvcf2.Variant): A variant object

        Returns:
            callers (dict): A dictionary on the format
            {'gatk': <filter>,'freebayes': <filter>,'samtools': <filter>}
    """
    relevant_callers = CALLERS[category]
    callers = {caller['id']: None for caller in relevant_callers}
    raw_info = variant.INFO.get('set')
    if raw_info:
        info = raw_info.split('-')
        for call in info:
            if call == 'FilteredInAll':
                for caller in callers:
                    callers[caller] = 'Filtered'
            elif call == 'Intersection':
                for caller in callers:
                    callers[caller] = 'Pass'
            elif 'filterIn' in call:
                for caller in callers:
                    if caller in call:
                        callers[caller] = 'Filtered'

            elif call in set(callers.keys()):
                callers[call] = 'Pass'
    # The following is parsing of a custom made merge
    other_info = variant.INFO.get('FOUND_IN')
    if other_info:
        for info in other_info.split(','):
            called_by = info.split('|')[0]
            callers[called_by] = 'Pass'

    return callers



import logging

from datetime import datetime
from pprint import pprint as pp

import click

from scout.load import (load_exons)

from scout.utils.handle import get_file_handle
from scout.utils.requests import (fetch_ensembl_exons)

LOG = logging.getLogger(__name__)

@click.command('exons', short_help='Load exons')
# @click.argument('infile', type=click.Path(exists=True))
@click.option('-b', '--build', 
    type=click.Choice(['37', '38']),
    default='37',
    show_default=True,
)
@click.pass_context
def exons(context, build):
    """Load exons into the scout database"""
    
    adapter = context.obj['adapter']
    
    start = datetime.now()
    # Test if there are any exons loaded
    
    nr_exons = adapter.exons(build=build).count()
    if nr_exons:
        LOG.warning("Dropping all exons ")
        adapter.drop_exons(build=build)
        LOG.info("Exons dropped")
    
    # Load the exons
    ensembl_exons = fetch_ensembl_exons(build=build)
    load_exons(adapter, ensembl_exons, build)

    adapter.update_indexes()
    
    LOG.info("Time to load exons: {0}".format(datetime.now() - start))




import pytest

from scout.parse.variant.callers import parse_callers
from scout.constants import CALLERS

def test_parse_callers(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that gatk and freebayes have passed
    variant.INFO['set'] = 'gatk-freebayes'

    # WHEN parsing the information
    callers = parse_callers(variant)

    #THEN the correct callers should be passed
    assert callers['gatk'] == 'Pass'
    assert callers['freebayes'] == 'Pass'
    assert callers['samtools'] == None

def test_parse_callers_only_one(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information about the variant callers
    variant.INFO['set'] = 'gatk'

    # WHEN parsing the information
    callers = parse_callers(variant)

    #THEN the correct callers should be passed
    assert callers['gatk'] == 'Pass'
    assert callers['freebayes'] == None
    assert callers['samtools'] == None

def test_parse_callers_complex(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information about the variant callers
    variant.INFO['set'] = 'gatk-filterInsamtools-freebayes'

    # WHEN parsing the information
    callers = parse_callers(variant)

    #THEN the correct output should be returned
    assert callers['gatk'] == 'Pass'
    assert callers['freebayes'] == 'Pass'
    assert callers['samtools'] == 'Filtered'

def test_parse_callers_intersection(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that all callers agree on Pass
    variant.INFO['set'] = 'Intersection'

    # WHEN parsing the information
    callers = parse_callers(variant)

    #THEN all callers should be passed
    assert callers['gatk'] == 'Pass'
    assert callers['freebayes'] == 'Pass'
    assert callers['samtools'] == 'Pass'

def test_parse_callers_filtered_all(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that all callers agree on filtered
    variant.INFO['set'] = 'FilteredInAll'

    # WHEN parsing the information
    callers = parse_callers(variant)

    #THEN all callers should be filtered
    assert callers['gatk'] == 'Filtered'
    assert callers['freebayes'] == 'Filtered'
    assert callers['samtools'] == 'Filtered'

def test_parse_sv_callers_filtered_all(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that all callers agree on filtered
    variant.INFO['set'] = 'FilteredInAll'

    # WHEN parsing the information
    callers = parse_callers(variant, category='sv')

    #THEN all callers should be filtered
    assert callers['cnvnator'] == 'Filtered'
    assert callers['delly'] == 'Filtered'
    assert callers['tiddit'] == 'Filtered'
    assert callers['manta'] == 'Filtered'

def test_parse_sv_callers_intersection(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that all callers agree on filtered
    variant.INFO['set'] = 'Intersection'

    # WHEN parsing the information
    callers = parse_callers(variant, category='sv')

    #THEN all callers should be filtered
    assert callers['cnvnator'] == 'Pass'
    assert callers['delly'] == 'Pass'
    assert callers['tiddit'] == 'Pass'
    assert callers['manta'] == 'Pass'

def test_parse_sv_callers_filterin_tiddit(cyvcf2_variant):
    variant = cyvcf2_variant
    # GIVEN information that all callers agree on filtered
    variant.INFO['set'] = 'manta-filterIntiddit'

    # WHEN parsing the information
    callers = parse_callers(variant, category='sv')

    #THEN all callers should be filtered
    assert callers['cnvnator'] == None
    assert callers['delly'] == None
    assert callers['tiddit'] == 'Filtered'
    assert callers['manta'] == 'Pass'


@pytest.mark.parametrize('category', ['snv', 'sv', 'cancer'])
def test_parse_callers_all(cyvcf2_variant, category):
    # GIVEN all callers called a cancer variant
    variant = cyvcf2_variant
    variant.INFO['set'] = 'Intersection'
    # WHEN parsing callers
    parsed_callers = parse_callers(variant, category=category)
    # THEN all callers should be in the dict
    legal_callers = set(caller['id'] for caller in CALLERS[category])
    assert legal_callers == set(parsed_callers.keys())



import logging

LOG = logging.getLogger(__name__)


def parse_hpo_phenotype(hpo_line):
    """Parse hpo phenotype
    
        Args:
            hpo_line(str): A iterable with hpo phenotype lines
    
        Yields:
            hpo_info(dict)
    """
    hpo_line = hpo_line.rstrip().split('\t')
    hpo_info = {}
    hpo_info['hpo_id'] = hpo_line[0]
    hpo_info['description'] = hpo_line[1]
    hpo_info['hgnc_symbol'] = hpo_line[3]
    
    return hpo_info

def parse_hpo_gene(hpo_line):
    """Parse hpo gene information
    
        Args:
            hpo_line(str): A iterable with hpo phenotype lines
    
        Yields:
            hpo_info(dict)
    """
    if not len(hpo_line) > 3:
        return {}
    hpo_line = hpo_line.rstrip().split('\t')
    hpo_info = {}
    hpo_info['hgnc_symbol'] = hpo_line[1]
    hpo_info['description'] = hpo_line[2]
    hpo_info['hpo_id'] = hpo_line[3]
    
    return hpo_info

def parse_hpo_disease(hpo_line):
    """Parse hpo disease line
    
        Args:
            hpo_line(str)
    """
    hpo_line = hpo_line.rstrip().split('\t')
    hpo_info = {}
    disease = hpo_line[0].split(':')
    
    hpo_info['source'] = disease[0]
    hpo_info['disease_nr'] = int(disease[1])
    hpo_info['hgnc_symbol'] = None
    hpo_info['hpo_term'] = None
    
    if len(hpo_line) >= 3:
        hpo_info['hgnc_symbol'] = hpo_line[2]

        if len(hpo_line) >= 4:
            hpo_info['hpo_term'] = hpo_line[3]
    
    
    return hpo_info

def parse_hpo_phenotypes(hpo_lines):
    """Parse hpo phenotypes
    
        Group the genes that a phenotype is associated to in 'genes'
    
    Args:
        hpo_lines(iterable(str)): A file handle to the hpo phenotypes file
    
    Returns:
        hpo_terms(dict): A dictionary with hpo_ids as keys and terms as values
    
        {
            <hpo_id>: {
                'hpo_id':str,
                'description': str,
                'hgnc_symbols': list(str), # [<hgnc_symbol>, ...]
                }
        }
    """
    hpo_terms = {}
    LOG.info("Parsing hpo phenotypes...")
    for index, line in enumerate(hpo_lines):
        if index > 0 and len(line) > 0:
            hpo_info = parse_hpo_phenotype(line)
            hpo_term = hpo_info['hpo_id']
            hgnc_symbol = hpo_info['hgnc_symbol']
            if hpo_term in hpo_terms:
                hpo_terms[hpo_term]['hgnc_symbols'].append(hgnc_symbol)
            else:
                hpo_terms[hpo_term] = {
                    'hpo_id':hpo_term,
                    'description': hpo_info['description'],
                    'hgnc_symbols': [hgnc_symbol]
                }
    LOG.info("Parsing done.")
    return hpo_terms

def parse_hpo_diseases(hpo_lines):
    """Parse hpo disease phenotypes
    
        Args:
            hpo_lines(iterable(str))
        
        Returns:
            diseases(dict): A dictionary with mim numbers as keys
    """
    diseases = {}
    LOG.info("Parsing hpo diseases...")
    for index, line in enumerate(hpo_lines):
        # First line is a header
        if index == 0:
            continue
        # Skip empty lines
        if not len(line) > 3:
            continue
        # Parse the info
        disease_info = parse_hpo_disease(line)
        # Skip the line if there where no info
        if not disease_info:
            continue
        disease_nr = disease_info['disease_nr']
        hgnc_symbol = disease_info['hgnc_symbol']
        hpo_term = disease_info['hpo_term']
        source = disease_info['source']
        disease_id = "{0}:{1}".format(source, disease_nr)
        
        if disease_id not in diseases:
            diseases[disease_id] = {
                'disease_nr': disease_nr,
                'source': source,
                'hgnc_symbols': set(),
                'hpo_terms': set(),
            }

        if hgnc_symbol:
            diseases[disease_id]['hgnc_symbols'].add(hgnc_symbol)
        if hpo_term:
            diseases[disease_id]['hpo_terms'].add(hpo_term)

    LOG.info("Parsing done.")
    return diseases
    

def parse_hpo_to_genes(hpo_lines):
    """Parse the map from hpo term to hgnc symbol
    
    Args:
        lines(iterable(str)):
    
    Yields:
        hpo_to_gene(dict): A dictionary with information on how a term map to a hgnc symbol
    """
    for line in hpo_lines:
        if line.startswith('#') or len(line) < 1:
            continue
        line = line.rstrip().split('\t')
        hpo_id = line[0]
        hgnc_symbol = line[3]
        
        yield {
            'hpo_id': hpo_id,
            'hgnc_symbol': hgnc_symbol
        }
    
    

def parse_hpo_genes(hpo_lines):
    """Parse HPO gene information
    
        Args:
            hpo_lines(iterable(str))
        
        Returns:
            diseases(dict): A dictionary with hgnc symbols as keys
    """
    LOG.info("Parsing HPO genes ...")
    genes = {}
    for index, line in enumerate(hpo_lines):
        # First line is header
        if index == 0:
            continue
        if len(line) < 5:
            continue
        gene_info = parse_hpo_gene(line)
        hgnc_symbol = gene_info['hgnc_symbol']
        description = gene_info['description']
        
        if hgnc_symbol not in genes:
            genes[hgnc_symbol] = {
                'hgnc_symbol': hgnc_symbol
            }
        
        gene = genes[hgnc_symbol]
        if description == 'Incomplete penetrance':
            gene['incomplete_penetrance'] = True
        if description == 'Autosomal dominant inheritance':
            gene['ad'] = True
        if description == 'Autosomal recessive inheritance':
            gene['ar'] = True
        if description == 'Mithochondrial inheritance':
            gene['mt'] = True
        if description == 'X-linked dominant inheritance':
            gene['xd'] = True
        if description == 'X-linked recessive inheritance':
            gene['xr'] = True
        if description == 'Y-linked inheritance':
            gene['x'] = True
        if description == 'X-linked inheritance':
            gene['y'] = True
    LOG.info("Parsing done.")
    return genes
    

def get_incomplete_penetrance_genes(hpo_lines):
    """Get a set with all genes that have incomplete penetrance according to HPO
    
    Args:
        hpo_lines(iterable(str))
    
    Returns:
        incomplete_penetrance_genes(set): A set with the hgnc symbols of all 
                                          genes with incomplete penetrance
        
    """
    genes = parse_hpo_genes(hpo_lines)
    incomplete_penetrance_genes = set()
    for hgnc_symbol in genes:
        if genes[hgnc_symbol].get('incomplete_penetrance'):
            incomplete_penetrance_genes.add(hgnc_symbol)
    return incomplete_penetrance_genes

def parse_hpo_obo(hpo_lines):
    """Parse a .obo formated hpo line"""
    term = {}
    for line in hpo_lines:
        if len(line) == 0:
            continue
        line = line.rstrip()
        # New term starts with [Term]
        if line == '[Term]':
            if term:
                yield term
            term = {}
        
        elif line.startswith('id'):
            term['hpo_id'] = line[4:]

        elif line.startswith('name'):
            term['description'] = line[6:]

        elif line.startswith('alt_id'):
            if 'aliases' not in term:
                term['aliases'] = []
            term['aliases'].append(line[8:])
        
        elif line.startswith('is_a'):
            if 'ancestors' not in term:
                term['ancestors'] = []
            term['ancestors'].append(line[6:16])

    if term:
        yield term
        

if __name__ == "__main__":
    import sys
    from pprint import pprint as pp
    from scout.utils.handle import get_file_handle
    
    file_handle = get_file_handle(sys.argv[1])
    
    # phenotypes = parse_hpo_phenotypes(file_handle)
    # for hpo_id in phenotypes:
    #     hpo_term = phenotypes[hpo_id]
    #     pp(hpo_term)

    diseases = parse_hpo_diseases(file_handle)
    for mim_id in diseases:
        disease = diseases[mim_id]
        pp(disease)

    # incomplete_genes = get_incomplete_penetrance_genes(file_handle)
    # for gene in incomplete_genes:
    #     print(gene)
    # genes = parse_hpo_genes(file_handle)
    # for hgnc_symbol in genes:
    #     gene = genes[hgnc_symbol]
    #     pp(gene)



"""
tests/adapter/mongo/test_comment_handling.py

We need to break out some tests from the event function.
"""

from pprint import pprint as pp

def test_specific_comment(adapter, institute_obj, case_obj, user_obj, variant_obj):
    content = 'specific comment for a variant'
    # GIVEN a populated database with a variant and no events
    adapter.variant_collection.insert_one(variant_obj)

    assert adapter.variant_collection.find().count() == 1
    assert adapter.event_collection.find().count() == 0

    # WHEN commenting a specific comment on a variant
    updated_variant = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='commentlink',
        variant=variant_obj,
        content=content,
        comment_level='specific'
    )
    # THEN assert that the events function returns the correct event.
    comments = adapter.events(institute_obj, case=case_obj, variant_id=variant_obj['variant_id'], 
                              comments=True)

    for comment in comments:
        assert comment['content'] == content
    
    ## THEN assert that when looking for the comments for a variant in a different case it will not be found
    other_case = case_obj
    other_case['_id'] = 'case2'

    comments = adapter.events(institute_obj, case=other_case, variant_id=variant_obj['variant_id'], 
                              comments=True)
    assert comments.count() == 0

def test_global_comment(adapter, institute_obj, case_obj, user_obj, variant_obj):
    content = 'global comment for a variant'
    # GIVEN a populated database with two variants from two cases and no events
    adapter.variant_collection.insert_one(variant_obj)
    
    
    assert adapter.variant_collection.find().count() == 1
    assert adapter.event_collection.find().count() == 0

    # WHEN commenting a global comment on a variant
    updated_variant = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='commentlink',
        variant=variant_obj,
        content=content,
        comment_level='global'
    )
    # THEN assert that the events function returns the correct event.
    comments = adapter.events(institute_obj, case=case_obj, variant_id=variant_obj['variant_id'], 
                           comments=True)

    for comment in comments:
        assert comment['content'] == content

    ## THEN assert that when looking for the comments for a variant in a different case one comment should be found
    other_case = case_obj
    other_case['_id'] = 'case2'

    comments = adapter.events(institute_obj, case=other_case, variant_id=variant_obj['variant_id'], 
                              comments=True)
    assert comments.count() == 1

def test_global_and_specific_comments_one_case(adapter, institute_obj, case_obj, user_obj, variant_obj):
    ## GIVEN an adapter with a variant and no events
    adapter.variant_collection.insert_one(variant_obj)

    ## WHEN adding a global and a specific comments
    global_content = 'global'
    specific_content = 'specific'
    
    global_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='globalcommentlink',
        variant=variant_obj,
        content=global_content,
        comment_level='global'
    )

    specific_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='specificcommentlink',
        variant=variant_obj,
        content=specific_content,
        comment_level='specific'
    )
    
    ## THEN assert that when fetching comments for a variant two events are returned
    comments = adapter.events(institute=institute_obj, case=case_obj, 
                              variant_id=variant_obj['variant_id'], comments=True)
    assert comments.count() == 2

def test_global_and_specific_comments_two_cases_same_institute(adapter, institute_obj, case_obj, user_obj, variant_obj):
    ## GIVEN an adapter with a variant and no events
    adapter.variant_collection.insert_one(variant_obj)

    ## WHEN adding a global and a specific comments for the first variant
    global_content = 'global'
    specific_content = 'specific'
    
    #Add a global comment
    global_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='globalcommentlink',
        variant=variant_obj,
        content=global_content,
        comment_level='global'
    )

    #Add a specific comment
    specific_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='globalcommentlink',
        variant=variant_obj,
        content=specific_content,
        comment_level='specific'
    )

    other_case = case_obj
    other_case['case_id'] = other_case['_id'] = 'case2'
    other_variant = variant_obj
    other_variant['case_id'] = other_case['case_id']
    other_variant['_id'] = other_variant['document_id'] = 'other_id'
    
    ## THEN assert that when fetching comments for other variant one global comment should be returned
    comments = adapter.events(institute=institute_obj, case=other_case, 
                              variant_id=other_variant['variant_id'], comments=True)
    
    assert comments.count() == 1

def test_global_and_specific_comments_two_cases_different_institutes(adapter, institute_obj, 
    case_obj, user_obj, variant_obj):
    ## GIVEN an adapter with a variant and no events
    adapter.variant_collection.insert_one(variant_obj)

    ## WHEN adding a global and a specific comments for the first variant
    global_content = 'global'
    specific_content = 'specific'
    
    #Add a global comment
    global_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='globalcommentlink',
        variant=variant_obj,
        content=global_content,
        comment_level='global'
    )

    #Add a specific comment
    specific_comment = adapter.comment(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='globalcommentlink',
        variant=variant_obj,
        content=specific_content,
        comment_level='specific'
    )

    other_case = case_obj
    other_case['case_id'] = other_case['_id'] = 'case2'
    other_variant = variant_obj
    other_variant['case_id'] = other_case['case_id']
    other_variant['_id'] = other_variant['document_id'] = 'other_id'
    other_institute = institute_obj
    other_institute['_id'] = 'inst2'

    ## THEN assert that when fetching comments for other variant one global comment should be returned

    comments = adapter.events(institute=other_institute, case=other_case, 
                              variant_id=other_variant['variant_id'], comments=True)
    
    assert comments.count() == 1



# -*- coding: utf-8 -*-


def event_rank(count):
    """Determine event ranking."""
    if count < 10:
        return 'constable'
    elif count < 100:
        return 'sergeant'
    elif count < 250:
        return 'inspector'
    elif count < 500:
        return 'superintendent'
    elif count < 1000:
        return 'commander'
    else:
        return 'commissioner'


def users(store):
    """Display a list of all users and which institutes they belong to."""
    user_objs = list(store.users())
    total_events = store.user_events().count()
    for user_obj in user_objs:
        user_obj['institutes'] = [store.institute(inst_id) for inst_id in user_obj['institutes']]
        user_obj['events'] = store.user_events(user_obj).count()
        user_obj['events_rank'] = event_rank(user_obj['events'])
    return dict(
        users=sorted(user_objs, key=lambda user: -user['events']),
        total_events=total_events,
    )



import logging
import click

LOG = logging.getLogger(__name__)

from scout.constants import (BUILDS, CHROMOSOMES)


@click.command('intervals', short_help='Show how many intervals that exists for each chromosome')
@click.option('-b', '--build',
              default='37',
              type=click.Choice(BUILDS)
              )
@click.pass_context
def intervals(context, build):
    """Show all indexes in the database"""
    LOG.info("Running scout view index")
    adapter = context.obj['adapter']

    intervals = adapter.get_coding_intervals(build)
    nr_intervals = 0
    longest = 0
    for chrom in CHROMOSOMES:
        for iv in intervals[chrom]:
            iv_len = iv.end - iv.begin
            if iv_len > longest:
                longest = iv_len
        int_nr = len(intervals.get(chrom, []))
        click.echo("{0}\t{1}".format(chrom, int_nr))
        nr_intervals += int_nr

    LOG.info("Total nr intervals:%s", nr_intervals)
    LOG.info("Total nr genes:%s", adapter.all_genes(build).count())
    LOG.info("Longest interval:%s", longest)



import copy
import logging
import datetime

from pprint import pprint as pp

from path import Path
from ped_parser import FamilyParser

from scout.exceptions import (PedigreeError, ConfigError)
from scout.constants import (PHENOTYPE_MAP, SEX_MAP, REV_SEX_MAP,
                             REV_PHENOTYPE_MAP)

from scout.parse.peddy import (parse_peddy_ped, parse_peddy_ped_check,
                               parse_peddy_sex_check)

log = logging.getLogger(__name__)


def parse_case_data(config=None, ped=None, owner=None, vcf_snv=None,
                    vcf_sv=None, vcf_cancer=None, vcf_str=None, peddy_ped=None,
                    peddy_sex=None, peddy_check=None, delivery_report=None, multiqc=None):
    """Parse all data necessary for loading a case into scout

    This can be done either by providing a VCF file and other information
    on the command line. Or all the information can be specified in a config file.
    Please see Scout documentation for further instructions.

    Args:
        config(dict): A yaml formatted config file
        ped(iterable(str)): A ped formatted family file
        owner(str): The institute that owns a case
        vcf_snv(str): Path to a vcf file
        vcf_str(str): Path to a VCF file
        vcf_sv(str): Path to a vcf file
        vcf_cancer(str): Path to a vcf file
        peddy_ped(str): Path to a peddy ped
        multiqc(str): Path to dir with multiqc information

    Returns:
        config_data(dict): Holds all the necessary information for loading
                           Scout
    """
    config_data = copy.deepcopy(config) or {}
    # Default the analysis date to now if not specified in load config
    if 'analysis_date' not in config_data:
        config_data['analysis_date'] = datetime.datetime.now()

    # If the family information is in a ped file we nned to parse that
    if ped:
        family_id, samples = parse_ped(ped)
        config_data['family'] = family_id
        config_data['samples'] = samples

    # Each case has to have a owner. If not provided in config file it needs to be given as a
    # argument
    if 'owner' not in config_data:
        if not owner:
            raise SyntaxError("Case has no owner")
        else:
            config_data['owner'] = owner

    if 'gene_panels' in config_data:
        # handle whitespace in gene panel names
        config_data['gene_panels'] = [panel.strip() for panel in
                                      config_data['gene_panels']]
        config_data['default_gene_panels'] = [panel.strip() for panel in
                                              config_data['default_gene_panels']]

    ##################### Add information from peddy if existing #####################
    config_data['peddy_ped'] = peddy_ped or config_data.get('peddy_ped')
    config_data['peddy_sex_check'] = peddy_sex or config_data.get('peddy_sex')
    config_data['peddy_ped_check'] = peddy_check or config_data.get('peddy_check')

    # This will add information from peddy to the individuals
    add_peddy_information(config_data)

    ##################### Add multiqc information #####################
    config_data['multiqc'] = multiqc or config_data.get('multiqc')

    config_data['vcf_snv'] = vcf_snv if vcf_snv else config_data.get('vcf_snv')
    config_data['vcf_sv'] = vcf_sv if vcf_sv else config_data.get('vcf_sv')
    config_data['vcf_str'] = vcf_str if vcf_str else config_data.get('vcf_str')
    log.debug("Config vcf_str set to {0}".format(config_data['vcf_str']))

    config_data['vcf_cancer'] = vcf_cancer if vcf_cancer else config_data.get('vcf_cancer')

    config_data['delivery_report'] = delivery_report if delivery_report else config_data.get('delivery_report')

    config_data['rank_model_version'] = config_data.get('rank_model_version')
    config_data['rank_score_threshold'] = config_data.get('rank_score_threshold', 0)

    return config_data


def add_peddy_information(config_data):
    """Add information from peddy outfiles to the individuals"""
    ped_info = {}
    ped_check = {}
    sex_check = {}
    relations = []

    if config_data.get('peddy_ped'):
        file_handle = open(config_data['peddy_ped'], 'r')
        for ind_info in parse_peddy_ped(file_handle):
            ped_info[ind_info['sample_id']] = ind_info

    if config_data.get('peddy_ped_check'):
        file_handle = open(config_data['peddy_ped_check'], 'r')
        for pair_info in parse_peddy_ped_check(file_handle):
            ped_check[(pair_info['sample_a'], pair_info['sample_b'])] = pair_info

    if config_data.get('peddy_sex_check'):
        file_handle = open(config_data['peddy_sex_check'], 'r')
        for ind_info in parse_peddy_sex_check(file_handle):
            sex_check[ind_info['sample_id']] = ind_info

    analysis_inds = {}
    for ind in config_data['samples']:
        ind_id = ind['sample_id']
        analysis_inds[ind_id] = ind

    for ind_id in analysis_inds:
        ind = analysis_inds[ind_id]
        # Check if peddy has inferred the ancestry
        if ind_id in ped_info:
            ind['predicted_ancestry'] = ped_info[ind_id].get('ancestry-prediction', 'UNKNOWN')
        # Check if peddy has inferred the sex
        if ind_id in sex_check:
            if sex_check[ind_id]['error']:
                ind['confirmed_sex'] = False
            else:
                ind['confirmed_sex'] = True

        # Check if peddy har confirmed parental relations
        for parent in ['mother', 'father']:
            # If we are looking at individual with parents
            if ind[parent] != '0':
                # Check if the child/parent pair is in peddy data
                for pair in ped_check:
                    if (ind_id in pair and ind[parent] in pair):
                        # If there is a parent error we mark that
                        if ped_check[pair]['parent_error']:
                            analysis_inds[ind[parent]]['confirmed_parent'] = False
                        else:
                            # Else if parent confirmation has not been done
                            if 'confirmed_parent' not in analysis_inds[ind[parent]]:
                                # Set confirmatio to True
                                analysis_inds[ind[parent]]['confirmed_parent'] = True

def parse_individual(sample):
    """Parse individual information

        Args:
            sample (dict)

        Returns:
            {
                'individual_id': str,
                'father': str,
                'mother': str,
                'display_name': str,
                'sex': str,
                'phenotype': str,
                'bam_file': str,
                'vcf2cytosure': str,
                'analysis_type': str,
                'capture_kits': list(str),
            }

    """
    ind_info = {}
    if 'sample_id' not in sample:
        raise PedigreeError("One sample is missing 'sample_id'")
    sample_id = sample['sample_id']
    # Check the sex
    if 'sex' not in sample:
        raise PedigreeError("Sample %s is missing 'sex'" % sample_id)
    sex = sample['sex']
    if sex not in REV_SEX_MAP:
        log.warning("'sex' is only allowed to have values from {}"
                    .format(', '.join(list(REV_SEX_MAP.keys()))))
        raise PedigreeError("Individual %s has wrong formated sex" % sample_id)

    # Check the phenotype
    if 'phenotype' not in sample:
        raise PedigreeError("Sample %s is missing 'phenotype'"
                            % sample_id)
    phenotype = sample['phenotype']
    if phenotype not in REV_PHENOTYPE_MAP:
        log.warning("'phenotype' is only allowed to have values from {}"
                    .format(', '.join(list(REV_PHENOTYPE_MAP.keys()))))
        raise PedigreeError("Individual %s has wrong formated phenotype" % sample_id)

    ind_info['individual_id'] = sample_id
    ind_info['display_name'] = sample.get('sample_name', sample['sample_id'])

    ind_info['sex'] = sex
    ind_info['phenotype'] = phenotype

    ind_info['father'] = sample.get('father')
    ind_info['mother'] = sample.get('mother')

    ind_info['confirmed_parent'] = sample.get('confirmed_parent')
    ind_info['confirmed_sex'] = sample.get('confirmed_sex')
    ind_info['predicted_ancestry'] = sample.get('predicted_ancestry')

    bam_file = sample.get('bam_path')
    if bam_file:
        ind_info['bam_file'] = bam_file

    mt_bam = sample.get('mt_bam')
    if mt_bam:
        ind_info['mt_bam'] = mt_bam

    analysis_type = sample.get('analysis_type')
    if analysis_type:
        ind_info['analysis_type'] = analysis_type

    ind_info['capture_kits'] = ([sample.get('capture_kit')]
                                if 'capture_kit' in sample else [])

    vcf2cytosure = sample.get('vcf2cytosure')
    if vcf2cytosure:
        ind_info['vcf2cytosure'] = vcf2cytosure

    return ind_info


def parse_individuals(samples):
    """Parse the individual information

        Reformat sample information to proper individuals

        Args:
            samples(list(dict))

        Returns:
            individuals(list(dict))
    """
    individuals = []
    if len(samples) == 0:
        raise PedigreeError("No samples could be found")

    ind_ids = set()
    for sample_info in samples:
        parsed_ind = parse_individual(sample_info)
        individuals.append(parsed_ind)
        ind_ids.add(parsed_ind['individual_id'])

    # Check if relations are correct
    for parsed_ind in individuals:
        father = parsed_ind['father']
        if (father and father != '0'):
            if father not in ind_ids:
                raise PedigreeError('father %s does not exist in family' % father)
        mother = parsed_ind['mother']
        if (mother and mother != '0'):
            if mother not in ind_ids:
                raise PedigreeError('mother %s does not exist in family' % mother)

    return individuals


def parse_case(config):
    """Parse case information from config or PED files.

    Args:
        config (dict): case config with detailed information

    Returns:
        dict: parsed case data
    """
    if 'owner' not in config:
        raise ConfigError("A case has to have a owner")

    if 'family' not in config:
        raise ConfigError("A case has to have a 'family'")

    individuals = parse_individuals(config['samples'])
    case_data = {
        'owner': config['owner'],
        'collaborators': [config['owner']],
        'case_id': config['family'],
        'display_name': config.get('family_name', config['family']),
        'genome_build': config.get('human_genome_build'),
        'rank_model_version': config.get('rank_model_version'),
        'rank_score_threshold': config.get('rank_score_threshold', 0),
        'analysis_date': config['analysis_date'],
        'individuals': individuals,
        'vcf_files': {
            'vcf_snv': config.get('vcf_snv'),
            'vcf_sv': config.get('vcf_sv'),
            'vcf_str': config.get('vcf_str'),
            'vcf_cancer': config.get('vcf_cancer'),
            'vcf_snv_research': config.get('vcf_snv_research'),
            'vcf_sv_research': config.get('vcf_sv_research'),
            'vcf_cancer_research': config.get('vcf_cancer_research'),
        },
        'default_panels': config.get('default_gene_panels', []),
        'gene_panels': config.get('gene_panels', []),
        'assignee': config.get('assignee'),
        'peddy_ped': config.get('peddy_ped'),
        'peddy_sex': config.get('peddy_sex'),
        'peddy_check': config.get('peddy_check'),
        'delivery_report': config.get('delivery_report'),
        'multiqc': config.get('multiqc'),
    }

    # add the pedigree figure, this is a xml file which is dumped in the db
    if 'madeline' in config:
        mad_path = Path(config['madeline'])
        if not mad_path.exists():
            raise ValueError("madeline path not found: {}".format(mad_path))
        with mad_path.open('r') as in_handle:
            case_data['madeline_info'] = in_handle.read()

    return case_data


def parse_ped(ped_stream, family_type='ped'):
    """Parse out minimal family information from a PED file.

    Args:
        ped_stream(iterable(str))
        family_type(str): Format of the pedigree information

    Returns:
        family_id(str), samples(list[dict])
    """
    pedigree = FamilyParser(ped_stream, family_type=family_type)

    if len(pedigree.families) != 1:
        raise PedigreeError("Only one case per ped file is allowed")

    family_id = list(pedigree.families.keys())[0]
    family = pedigree.families[family_id]

    samples = [{
        'sample_id': ind_id,
        'father': individual.father,
        'mother': individual.mother,
        # Convert sex to human readable
        'sex': SEX_MAP[individual.sex],
        'phenotype': PHENOTYPE_MAP[int(individual.phenotype)],
    } for ind_id, individual in family.individuals.items()]

    return family_id, samples



# -*- coding: utf-8 -*-

from pymongo.errors import (ConnectionFailure)


from flask_debugtoolbar import DebugToolbarExtension
toolbar = DebugToolbarExtension()

from flask_bootstrap import Bootstrap
bootstrap = Bootstrap()

from scout.adapter import MongoAdapter
store = MongoAdapter()

from flask_login import LoginManager
from flask_oauthlib.client import OAuth
login_manager = LoginManager()
oauth = OAuth()
# use Google as remote application
# you must configure 3 values from Google APIs console
# https://code.google.com/apis/console
google = oauth.remote_app('google', app_key='GOOGLE')

from flask_mail import Mail
mail = Mail()


from loqusdb.plugins import MongoAdapter as LoqusDBMongoAdapter

from scout.adapter.client import get_connection


class LoqusDB(LoqusDBMongoAdapter):
    def init_app(self, app):
        """Initialize from Flask."""
        self.connect(**app.config['LOQUSDB_SETTINGS'])

    def case_count(self):
        return self.db.case.find({}).count()

class MongoDB(object):
    
    def init_app(self, app):
        """Initialize from flask"""
        uri = app.config.get("MONGO_URI", None)
        
        db_name = app.config.get("MONGO_DBNAME", 'scout')
        
        try:
            client = get_connection(
                host = app.config.get("MONGO_HOST", 'localhost'),
                port=app.config.get("MONGO_PORT", 27017), 
                username=app.config.get("MONGO_USERNAME", None), 
                password=app.config.get("MONGO_PASSWORD", None),
                uri=uri, 
                mongodb= db_name
            )
        except ConnectionFailure:
            context.abort()

        app.config["MONGO_DATABASE"] = client[db_name]

        app.config['MONGO_CLIENT'] = client
        

loqusdb = LoqusDB()
mongo = MongoDB()



import logging

import click

LOG = logging.getLogger(__name__)


@click.command('collections', short_help='Display all collections')
@click.pass_context
def collections(context):
    """Show all collections in the database"""
    LOG.info("Running scout view collections")

    adapter = context.obj['adapter']

    for collection_name in adapter.collections():
        click.echo(collection_name)



# clinvar Variant sheet
CLINVAR_HEADER = {
    'local_id' : '##Local ID',
    'linking_id' : 'Linking ID',
    'gene_symbol' : 'Gene symbol',
    'ref_seq' : 'Reference sequence',
    'hgvs' : 'HGVS',
    'chromosome' : 'Chromosome',
    'start' : 'Start',
    'stop' : 'Stop',
    'ref' : 'Reference allele',
    'alt' : 'Alternate allele',
    'var_type' : 'Variant type',
    'ncopy' : 'Copy number',
    'ref_copy' : 'Reference copy number',
    'breakpoint1' : 'Breakpoint 1',
    'breakpoint2' : 'Breakpoint 2',
    'outer_start' : 'Outer start',
    'inner_start' : 'Inner start',
    'inner_stop' : 'Inner stop',
    'outer_stop' : 'Outer stop',
    'variations_ids' : 'Variation identifiers',
    'condition_id_type' : 'Condition ID type', #default = 'HPO'
    'condition_id_value' : 'Condition ID value',
    'condition_comment' : 'Condition comment',
    'clinsig' : 'Clinical significance',
    'last_evaluated' : 'Date last evaluated',
    'variant_comment' : 'Comment on variant',
    'assertion_method' : 'Assertion method',
    'inheritance_mode' : 'Mode of inheritance',
    'clinsig_cit' : 'Clinical significance citations',
    'clinsig_comment' : 'Comment on clinical significance',
    'drug_response' : 'Drug response condition',
    'funct_conseq' : 'Functional consequence',
    'funct_conseq_comment' : 'Comment on functional consequence'
}

# clinvar CaseData sheet
CASEDATA_HEADER = {
    'linking_id' : 'Linking ID',
    'individual_id' : 'Individual ID',
    'collection_method' : 'Collection method', # default = 'clinical testing'
    'allele_origin' : 'Allele origin',
    'is_affected' : 'Affected status',
    'sv_analysis_method' : 'Structural variant method/analysis type',
    'clin_features' : 'Clinical features',
    'clin_features_comment' : 'Comment on clinical features',
    'tissue' : 'Tissue',
    'sex' : 'Sex',
    'age' : 'Age',
    'ethnicity' : 'Population Group/Ethnicity',
    'fam_history' : 'Family history',
    'is_proband' : 'Proband',
    'is_secondary_finding' : 'Secondary finding',
    'is_mosaic' : 'Mosaicism',
    'zygosity' : 'Zygosity',
    'co_occurr_gene' : 'Co-occurrences, same gene',
    'co_occurr_other' : 'Co-occurrence, other genes',
    'evidence_cit' : 'Evidence citations',
    'platform_type' : 'Platform type',
    'platform_name' : 'Platform name',
    'method' : 'Method',
    'method_purpose' : 'Method purpose',
    'method_cit' : 'Method citations',
    'testing_lab' : 'Testing laboratory',
    'reported_at' : 'Date variant was reported to submitter'
}



import os
import click
import logging
import datetime

from xlsxwriter import Workbook

from scout.constants import MT_EXPORT_HEADER
from scout.export.variant import export_mt_variants

LOG = logging.getLogger(__name__)

@click.command('mt_report', short_help='Mitochondrial variants Excel report')
@click.option('--case_id',
              help='Case id to search for',
              required=True
)
@click.option('--outpath',
              help='Path to output file'
)
@click.option('--test',
              help='Use this flag to test the function',
              is_flag=True
)
@click.pass_context
def mt_report(context, case_id, test, outpath=None):
    """Export all mitochondrial variants for each sample of a case
       and write them to an excel file

        Args:
            adapter(MongoAdapter)
            case_id(str)
            test(bool): True if the function is called for testing purposes
            outpath(str): path to output file

        Returns:
            written_files(int): number of written or simulated files
    """
    LOG.info('exporting mitochondrial variants for case "{}"'.format(case_id))

    adapter = context.obj['adapter']
    query = {'chrom':'MT'}

    case_obj = adapter.case(case_id=case_id)

    if not case_obj:
        LOG.warning('Could not find a scout case with id "{}". No report was created.'.format(case_id))
        context.abort()

    samples = case_obj.get('individuals')
    mt_variants = list(adapter.variants(case_id=case_id, query=query, nr_of_variants= -1, sort_key='position'))
    if not mt_variants:
        LOG.warning('There are no MT variants associated to case {} in database!'.format(case_id))
        context.abort()

    today = datetime.datetime.now().strftime('%Y-%m-%d')

    # set up outfolder
    if not outpath:
        outpath = str(os.getcwd())

    # get document lines for each of the cases's individuals
    # Write excel document for each sample in case
    written_files = 0

    for sample in samples:
        sample_id = sample['individual_id']
        sample_lines = export_mt_variants(variants=mt_variants, sample_id=sample_id)

        # set up document name
        document_name = '.'.join([case_obj['display_name'], sample_id, today]) + '.xlsx'
        workbook = Workbook(os.path.join(outpath,document_name))
        Report_Sheet = workbook.add_worksheet()

        if test and sample_lines and workbook:
            written_files +=1
            continue

        # Write the column header
        row = 0
        for col,field in enumerate(MT_EXPORT_HEADER):
            Report_Sheet.write(row,col,field)

        # Write variant lines, after header (start at line 1)
        for row, line in enumerate(sample_lines,1): # each line becomes a row in the document
            for col, field in enumerate(line): # each field in line becomes a cell
                Report_Sheet.write(row,col,field)
        workbook.close()

        if os.path.exists(os.path.join(outpath,document_name)):
            written_files += 1

    if test:
        LOG.info("Number of excel files that can be written to folder {0}: {1}".format(outpath, written_files))
    else:
        LOG.info("Number of excel files written to folder {0}: {1}".format(outpath, written_files))
    return written_files



# -*- coding: utf-8 -*-
import logging
import urllib.parse

LOG = logging.getLogger(__name__)

from scout.constants import (CHROMOSOMES, CHROMOSOME_INTEGERS, CALLERS)

def export_variants(adapter, collaborator, document_id=None, case_id=None):
    """Export causative variants for a collaborator

    Args:
        adapter(MongoAdapter)
        collaborator(str)
        document_id(str): Search for a specific variant
        case_id(str): Search causative variants for a case

    Yields:
        variant_obj(scout.Models.Variant): Variants marked as causative ordered by position.
    """

    # Store the variants in a list for sorting
    variants = []
    if document_id:
        yield adapter.variant(document_id)
        return

    variant_ids = adapter.get_causatives(
        institute_id=collaborator,
        case_id=case_id
        )
    ##TODO add check so that same variant is not included more than once
    for document_id in variant_ids:

        variant_obj = adapter.variant(document_id)
        chrom = variant_obj['chromosome']
        # Convert chromosome to integer for sorting
        chrom_int = CHROMOSOME_INTEGERS.get(chrom)
        if not chrom_int:
            LOG.info("Unknown chromosome %s", chrom)
            continue

        # Add chromosome and position to prepare for sorting
        variants.append((chrom_int, variant_obj['position'], variant_obj))

    # Sort varants based on position
    variants.sort(key=lambda x: (x[0], x[1]))

    for variant in variants:
        variant_obj = variant[2]
        yield variant_obj


def export_verified_variants(aggregate_variants):
    """Create the lines for an excel file with verified variants for
        an institute

        Args:
            aggregate_variants(list): a list of variants with aggregates case data

        Returns:
            document_lines(list): list of lines to include in the document
    """
    document_lines = []
    for variant in aggregate_variants:
        line = [] # line elements corespond to contants.variants_export.VERIFIED_VARIANTS_HEADER
        line.append(variant['_id']) # variant database ID
        line.append(variant['category'])
        line.append(variant['variant_type'])
        line.append(variant['display_name'][:30]) # variant display name
        line.append(variant.get('validation'))
        case_name = variant['case_obj']['display_name']  # case display name
        line.append(case_name)
        line.append(variant['institute'])
        line.append(''.join(['chr',variant['chromosome'],':',str(variant['position'])])) # position
        line.append('>'.join([variant.get('reference')[:10],variant.get('alternative')[:10]])) # change
        genes = []
        prot_effect = []
        for gene in variant.get('genes'): # this will be a unique long field in the document
            genes.append(gene.get('hgnc_symbol',''))
            for transcript in gene.get('transcripts'):
                if transcript.get('is_canonical') and transcript.get('protein_sequence_name'):
                    prot_effect.append(urllib.parse.unquote(transcript.get('protein_sequence_name')))
        line.append(','.join(genes))
        line.append(','.join(prot_effect))

        # get variant callers:
        callers = [] # a list of dictionaries
        call_results = []
        if variant['category'] == 'snv':
            callers = CALLERS['snv']
        elif variant['category'] == 'sv':
            callers = CALLERS['sv']

        for caller_obj in callers:
            caller_id = caller_obj['id']
            if variant.get(caller_id):
                call_results.append(':'.join([caller_id, variant.get(caller_id)]))

        line.append(','.join(call_results))
        line.append(variant.get('rank_score'))
        line.append(variant.get('cadd_score'))

        # get genotype and allele depth for each sample
        samples = []
        gtypes = []
        for sample in variant['samples']:
            case_individual = next(ind for ind in variant['case_obj']['individuals'] if ind['individual_id'] == sample['sample_id'])
            if case_individual['phenotype'] == 2:
                samples.append(''.join([sample['display_name'],'(A)'])) # label sample as affected
            else:
                samples.append(sample['display_name'])
            depth = ''.join([ '(', str(sample['allele_depths'][0]), '/', str(sample['allele_depths'][1]), ')' ])
            gtypes.append(' '.join([ sample['genotype_call'], depth ]))

        line.append(','.join(samples))
        line.append(','.join(gtypes))

        # Build local link to variant:
        local_link = '/'.join([ '', variant['institute'], case_name, variant['_id'] ])
        line.append(local_link)

        document_lines.append(line)
    return document_lines


def export_mt_variants(variants, sample_id):
    """Export mitochondrial variants for a case to create a MT excel report

    Args:
        variants(list): all MT variants for a case, sorted by position
        sample_id(str) : the id of a sample within the case

    Returns:
        document_lines(list): list of lines to include in the document
    """
    document_lines = []
    for variant in variants:
        line = []
        position = variant.get('position')
        change = '>'.join([variant.get('reference'),variant.get('alternative')])
        line.append(position)
        line.append(change)
        line.append(str(position)+change)
        genes = []
        prot_effect = []
        for gene in variant.get('genes'):
            genes.append(gene.get('hgnc_symbol',''))
            for transcript in gene.get('transcripts'):
                if transcript.get('is_canonical') and transcript.get('protein_sequence_name'):
                    prot_effect.append(urllib.parse.unquote(transcript.get('protein_sequence_name')))
        line.append(','.join(genes))
        line.append(','.join(prot_effect))
        ref_ad = ''
        alt_ad = ''
        for sample in variant['samples']:
            if sample.get('sample_id') == sample_id:
                ref_ad = sample['allele_depths'][0]
                alt_ad = sample['allele_depths'][1]
        line.append(ref_ad)
        line.append(alt_ad)
        document_lines.append(line)
    return document_lines
